@@ -23,8 +23,7 @@
 #define NET_FLOW6	8
 #define NET_IP6_SADR	9
 #define NET_MPLS	10
-#define NET_ASPA	11
-#define NET_MAX		12
+#define NET_MAX		11
 
 #define NB_IP4		(1 << NET_IP4)
 #define NB_IP6		(1 << NET_IP6)
@@ -36,7 +35,6 @@
 #define NB_FLOW6	(1 << NET_FLOW6)
 #define NB_IP6_SADR	(1 << NET_IP6_SADR)
 #define NB_MPLS		(1 << NET_MPLS)
-#define NB_ASPA		(1 << NET_ASPA)
 
 #define NB_IP		(NB_IP4 | NB_IP6)
 #define NB_VPN		(NB_VPN4 | NB_VPN6)
@@ -51,7 +49,7 @@ typedef struct net_addr {
   u8 pxlen;
   u16 length;
   u8 data[20];
-  u32 align[0];
+  u64 align[0];
 } net_addr;
 
 typedef struct net_addr_ip4 {
@@ -73,7 +71,7 @@ typedef struct net_addr_vpn4 {
   u8 pxlen;
   u16 length;
   ip4_addr prefix;
-  vpn_rd rd;
+  u64 rd;
 } net_addr_vpn4;
 
 typedef struct net_addr_vpn6 {
@@ -82,7 +80,7 @@ typedef struct net_addr_vpn6 {
   u16 length;
   ip6_addr prefix;
   u32 padding;
-  vpn_rd rd;
+  u64 rd;
 } net_addr_vpn6;
 
 typedef struct net_addr_roa4 {
@@ -126,13 +124,6 @@ typedef struct net_addr_mpls {
   u32 label;
 } net_addr_mpls;
 
-typedef struct net_addr_aspa {
-  u8 type;
-  u8 pxlen;
-  u16 length;
-  u32 asn;
-} net_addr_aspa;
-
 typedef struct net_addr_ip6_sadr {
   u8 type;
   u8 dst_pxlen;
@@ -154,7 +145,6 @@ typedef union net_addr_union {
   net_addr_flow6 flow6;
   net_addr_ip6_sadr ip6_sadr;
   net_addr_mpls mpls;
-  net_addr_aspa aspa;
 } net_addr_union;
 
 
@@ -193,9 +183,6 @@ extern const u16 net_max_text_length[];
 #define NET_ADDR_IP6_SADR(dst_prefix,dst_pxlen,src_prefix,src_pxlen) \
   ((net_addr_ip6_sadr) { NET_IP6_SADR, dst_pxlen, sizeof(net_addr_ip6_sadr), dst_prefix, src_pxlen, src_prefix })
 
-#define NET_ADDR_ASPA(asn) \
-  ((net_addr_aspa) { NET_ASPA, 32, sizeof(net_addr_aspa), asn })
-
 #define NET_ADDR_MPLS(label) \
   ((net_addr_mpls) { NET_MPLS, 20, sizeof(net_addr_mpls), label })
 
@@ -206,10 +193,10 @@ static inline void net_fill_ip4(net_addr *a, ip4_addr prefix, uint pxlen)
 static inline void net_fill_ip6(net_addr *a, ip6_addr prefix, uint pxlen)
 { *(net_addr_ip6 *)a = NET_ADDR_IP6(prefix, pxlen); }
 
-static inline void net_fill_vpn4(net_addr *a, ip4_addr prefix, uint pxlen, vpn_rd rd)
+static inline void net_fill_vpn4(net_addr *a, ip4_addr prefix, uint pxlen, u64 rd)
 { *(net_addr_vpn4 *)a = NET_ADDR_VPN4(prefix, pxlen, rd); }
 
-static inline void net_fill_vpn6(net_addr *a, ip6_addr prefix, uint pxlen, vpn_rd rd)
+static inline void net_fill_vpn6(net_addr *a, ip6_addr prefix, uint pxlen, u64 rd)
 { *(net_addr_vpn6 *)a = NET_ADDR_VPN6(prefix, pxlen, rd); }
 
 static inline void net_fill_roa4(net_addr *a, ip4_addr prefix, uint pxlen, uint max_pxlen, u32 asn)
@@ -224,9 +211,6 @@ static inline void net_fill_ip6_sadr(net_addr *a, ip6_addr dst_prefix, uint dst_
 static inline void net_fill_mpls(net_addr *a, u32 label)
 { *(net_addr_mpls *)a = NET_ADDR_MPLS(label); }
 
-static inline void net_fill_aspa(net_addr *a, u32 asn)
-{ *(net_addr_aspa *)a = NET_ADDR_ASPA(asn); }
-
 static inline void net_fill_ipa(net_addr *a, ip_addr prefix, uint pxlen)
 {
   if (ipa_is_ip4(prefix))
@@ -285,9 +269,6 @@ static inline int net_is_roa(const net_addr *a)
 static inline int net_is_flow(const net_addr *a)
 { return (a->type == NET_FLOW4) || (a->type == NET_FLOW6); }
 
-static inline int net_is_aspa(const net_addr *a)
-{ return (a->type == NET_ASPA); }
-
 static inline int net_is_sadr(const net_addr *a)
 { return (a->type == NET_IP6_SADR); }
 
@@ -315,7 +296,6 @@ static inline ip_addr net_prefix(const net_addr *a)
     return ipa_from_ip6(net6_prefix(a));
 
   case NET_MPLS:
-  case NET_ASPA:
   default:
     return IPA_NONE;
   }
@@ -340,7 +320,7 @@ static inline uint net_pxlen(const net_addr *a)
 
 ip_addr net_pxmask(const net_addr *a);
 
-static inline vpn_rd net_rd(const net_addr *a)
+static inline u64 net_rd(const net_addr *a)
 {
   switch (a->type)
   {
@@ -349,7 +329,7 @@ static inline vpn_rd net_rd(const net_addr *a)
   case NET_VPN6:
     return ((net_addr_vpn6 *)a)->rd;
   }
-  return RD_NONE;
+  return 0;
 }
 
 
@@ -386,9 +366,6 @@ static inline int net_equal_ip6_sadr(const net_addr_ip6_sadr *a, const net_addr_
 static inline int net_equal_mpls(const net_addr_mpls *a, const net_addr_mpls *b)
 { return !memcmp(a, b, sizeof(net_addr_mpls)); }
 
-static inline int net_equal_aspa(const net_addr_aspa *a, const net_addr_aspa *b)
-{ return !memcmp(a, b, sizeof(net_addr_aspa)); }
-
 
 static inline int net_equal_prefix_roa4(const net_addr_roa4 *a, const net_addr_roa4 *b)
 { return ip4_equal(a->prefix, b->prefix) && (a->pxlen == b->pxlen); }
@@ -410,10 +387,10 @@ static inline int net_zero_ip6(const net_addr_ip6 *a)
 { return !a->pxlen && ip6_zero(a->prefix); }
 
 static inline int net_zero_vpn4(const net_addr_vpn4 *a)
-{ return !a->pxlen && ip4_zero(a->prefix) && rd_zero(a->rd); }
+{ return !a->pxlen && ip4_zero(a->prefix) && !a->rd; }
 
 static inline int net_zero_vpn6(const net_addr_vpn6 *a)
-{ return !a->pxlen && ip6_zero(a->prefix) && rd_zero(a->rd); }
+{ return !a->pxlen && ip6_zero(a->prefix) && !a->rd; }
 
 static inline int net_zero_roa4(const net_addr_roa4 *a)
 { return !a->pxlen && ip4_zero(a->prefix) && !a->max_pxlen && !a->asn; }
@@ -430,9 +407,6 @@ static inline int net_zero_flow6(const net_addr_flow6 *a)
 static inline int net_zero_mpls(const net_addr_mpls *a)
 { return !a->label; }
 
-static inline int net_zero_aspa(const net_addr_aspa *a)
-{ return !a->asn; }
-
 
 static inline int net_compare_ip4(const net_addr_ip4 *a, const net_addr_ip4 *b)
 { return ip4_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
@@ -441,10 +415,10 @@ static inline int net_compare_ip6(const net_addr_ip6 *a, const net_addr_ip6 *b)
 { return ip6_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
 
 static inline int net_compare_vpn4(const net_addr_vpn4 *a, const net_addr_vpn4 *b)
-{ return rd_compare(a->rd, b->rd) ?: ip4_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
+{ return u64_cmp(a->rd, b->rd) ?: ip4_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
 
 static inline int net_compare_vpn6(const net_addr_vpn6 *a, const net_addr_vpn6 *b)
-{ return rd_compare(a->rd, b->rd) ?: ip6_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
+{ return u64_cmp(a->rd, b->rd) ?: ip6_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen); }
 
 static inline int net_compare_roa4(const net_addr_roa4 *a, const net_addr_roa4 *b)
 { return ip4_compare(a->prefix, b->prefix) ?: uint_cmp(a->pxlen, b->pxlen) ?: uint_cmp(a->max_pxlen, b->max_pxlen) ?: uint_cmp(a->asn, b->asn); }
@@ -468,9 +442,6 @@ static inline int net_compare_ip6_sadr(const net_addr_ip6_sadr *a, const net_add
 static inline int net_compare_mpls(const net_addr_mpls *a, const net_addr_mpls *b)
 { return uint_cmp(a->label, b->label); }
 
-static inline int net_compare_aspa(const net_addr_aspa *a, const net_addr_aspa *b)
-{ return uint_cmp(a->asn, b->asn); }
-
 int net_compare(const net_addr *a, const net_addr *b);
 
 
@@ -507,54 +478,40 @@ static inline void net_copy_ip6_sadr(net_addr_ip6_sadr *dst, const net_addr_ip6_
 static inline void net_copy_mpls(net_addr_mpls *dst, const net_addr_mpls *src)
 { memcpy(dst, src, sizeof(net_addr_mpls)); }
 
-static inline void net_copy_aspa(net_addr_aspa *dst, const net_addr_aspa *src)
-{ memcpy(dst, src, sizeof(net_addr_aspa)); }
 
-
-static inline u32 px4_hash(ip4_addr prefix, u32 pxlen)
-{ return ip4_hash(prefix) ^ (pxlen << 26); }
-
-static inline u32 px6_hash(ip6_addr prefix, u32 pxlen)
-{ return ip6_hash(prefix) ^ (pxlen << 26); }
+/* XXXX */
+static inline u32 u64_hash(u64 a)
+{ return u32_hash(a); }
 
 static inline u32 net_hash_ip4(const net_addr_ip4 *n)
-{ return px4_hash(n->prefix, n->pxlen); }
+{ return ip4_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_ip6(const net_addr_ip6 *n)
-{ return px6_hash(n->prefix, n->pxlen); }
+{ return ip6_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_vpn4(const net_addr_vpn4 *n)
-{
-  u64 acc = ip4_hash0(n->prefix, HASH_PARAM, 0) ^ (n->pxlen << 26);
-  return hash_value(rd_hash0(n->rd, HASH_PARAM, acc));
-}
+{ return ip4_hash(n->prefix) ^ ((u32) n->pxlen << 26) ^ u64_hash(n->rd); }
 
 static inline u32 net_hash_vpn6(const net_addr_vpn6 *n)
-{
-  u64 acc = ip6_hash0(n->prefix, HASH_PARAM, 0) ^ (n->pxlen << 26);
-  return hash_value(rd_hash0(n->rd, HASH_PARAM, acc));
-}
+{ return ip6_hash(n->prefix) ^ ((u32) n->pxlen << 26) ^ u64_hash(n->rd); }
 
 static inline u32 net_hash_roa4(const net_addr_roa4 *n)
-{ return px4_hash(n->prefix, n->pxlen); }
+{ return ip4_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_roa6(const net_addr_roa6 *n)
-{ return px6_hash(n->prefix, n->pxlen); }
+{ return ip6_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_flow4(const net_addr_flow4 *n)
-{ return px4_hash(n->prefix, n->pxlen); }
+{ return ip4_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_flow6(const net_addr_flow6 *n)
-{ return px6_hash(n->prefix, n->pxlen); }
+{ return ip6_hash(n->prefix) ^ ((u32) n->pxlen << 26); }
 
 static inline u32 net_hash_ip6_sadr(const net_addr_ip6_sadr *n)
-{ return px6_hash(n->dst_prefix, n->dst_pxlen); }
+{ return net_hash_ip6((net_addr_ip6 *) n); }
 
 static inline u32 net_hash_mpls(const net_addr_mpls *n)
-{ return u32_hash(n->label); }
-
-static inline u32 net_hash_aspa(const net_addr_aspa *n)
-{ return u32_hash(n->asn); }
+{ return n->label; }
 
 u32 net_hash(const net_addr *a);
 
@@ -605,9 +562,6 @@ static inline int net_validate_flow6(const net_addr_flow6 *n)
 static inline int net_validate_mpls(const net_addr_mpls *n)
 { return n->label < (1 << 20); }
 
-static inline int net_validate_aspa(const net_addr_aspa *n)
-{ return n->asn > 0; }
-
 static inline int net_validate_ip6_sadr(const net_addr_ip6_sadr *n)
 { return net_validate_px6(n->dst_prefix, n->dst_pxlen) && net_validate_px6(n->src_prefix, n->src_pxlen); }
 
@@ -636,8 +590,8 @@ void net_normalize(net_addr *N);
 
 
 int net_classify(const net_addr *N);
-int net_format(const net_addr *N, char *buf, int buflen) ACCESS_WRITE(2, 3);
-int rd_format(const vpn_rd rd, char *buf, int buflen) ACCESS_WRITE(2, 3);
+int net_format(const net_addr *N, char *buf, int buflen);
+int rd_format(const u64 rd, char *buf, int buflen);
 
 static inline int ipa_in_px4(ip4_addr a, ip4_addr prefix, uint pxlen)
 { return ip4_zero(ip4_and(ip4_xor(a, prefix), ip4_mkmask(pxlen))); }
