@@ -596,7 +596,6 @@ babel_update_cost(struct babel_neighbor *nbr)
   switch (cf->type)
   {
   case BABEL_IFACE_TYPE_WIRED:
-  case BABEL_IFACE_TYPE_TUNNEL:
     /* k-out-of-j selection - Appendix 2.1 in the RFC. */
 
     /* Link is bad if less than cf->limit/16 of expected hellos were received */
@@ -625,24 +624,6 @@ babel_update_cost(struct babel_neighbor *nbr)
     break;
   }
 
-  if (cf->rtt_cost && nbr->srtt > cf->rtt_min)
-  {
-    uint rtt_cost = cf->rtt_cost;
-
-    if (nbr->srtt < cf->rtt_max)
-    {
-      uint rtt_interval = cf->rtt_max TO_US - cf->rtt_min TO_US;
-      uint rtt_diff = (nbr->srtt TO_US - cf->rtt_min TO_US);
-
-      rtt_cost = (rtt_cost * rtt_diff) / rtt_interval;
-    }
-
-    txcost = MIN(txcost + rtt_cost, BABEL_INFINITY);
-
-    TRACE(D_EVENTS, "Added RTT cost %u to nbr %I on %s with srtt %t ms",
-	  rtt_cost, nbr->addr, nbr->ifa->iface->name, nbr->srtt * 1000);
-  }
-
 done:
   /* If RX cost changed, send IHU with next Hello */
   if (rxcost != nbr->rxcost)
@@ -873,12 +854,6 @@ babel_build_ihu(union babel_msg *msg, struct babel_iface *ifa, struct babel_neig
   msg->ihu.rxcost = n->rxcost;
   msg->ihu.interval = ifa->cf->ihu_interval;
 
-  if (n->last_tstamp_rcvd && ifa->cf->rtt_send)
-  {
-    msg->ihu.tstamp = n->last_tstamp;
-    msg->ihu.tstamp_rcvd = n->last_tstamp_rcvd TO_US;
-  }
-
   TRACE(D_PACKETS, "Sending IHU for %I with rxcost %d interval %t",
         msg->ihu.addr, msg->ihu.rxcost, (btime) msg->ihu.interval);
 }
@@ -918,9 +893,6 @@ babel_send_hello(struct babel_iface *ifa, uint interval)
   msg.hello.seqno = ifa->hello_seqno++;
   msg.hello.interval = interval ?: ifa->cf->hello_interval;
 
-  if (ifa->cf->rtt_send)
-    msg.hello.tstamp = 1; /* real timestamp will be set on TLV write */
-
   TRACE(D_PACKETS, "Sending hello on %s with seqno %d interval %t",
 	ifa->ifname, msg.hello.seqno, (btime) msg.hello.interval);
 
@@ -1015,7 +987,7 @@ babel_send_update_(struct babel_iface *ifa, btime changed, struct fib *rtable)
 
     /* Our own seqno might have changed, in which case we update the routes we
        originate. */
-    if ((e->router_id == p->router_id) && gt_mod64k(p->update_seqno, e->seqno))
+    if ((e->router_id == p->router_id) && (e->seqno < p->update_seqno))
     {
       e->seqno = p->update_seqno;
       e->updated = current_time();
@@ -1227,26 +1199,14 @@ babel_handle_hello(union babel_msg *m, struct babel_iface *ifa)
 	msg->seqno, (btime) msg->interval);
 
   struct babel_neighbor *n = babel_get_neighbor(ifa, msg->sender);
-  struct babel_iface_config *cf = n->ifa->cf;
   int first_hello = !n->hello_cnt;
 
-  if (msg->tstamp)
-  {
-    n->last_tstamp = msg->tstamp;
-    n->last_tstamp_rcvd = msg->pkt_received;
-  }
   babel_update_hello_history(n, msg->seqno, msg->interval);
   babel_update_cost(n);
 
   /* Speed up session establishment by sending IHU immediately */
   if (first_hello)
-  {
-    /* if using RTT, all IHUs must be paired with hellos */
-    if(cf->rtt_send)
-      babel_send_hello(ifa, 0);
-    else
-      babel_send_ihu(ifa, n);
-  }
+    babel_send_ihu(ifa, n);
 }
 
 void
@@ -1265,39 +1225,6 @@ babel_handle_ihu(union babel_msg *m, struct babel_iface *ifa)
   struct babel_neighbor *n = babel_get_neighbor(ifa, msg->sender);
   n->txcost = msg->rxcost;
   n->ihu_expiry = current_time() + BABEL_IHU_EXPIRY_FACTOR(msg->interval);
-
-  if (msg->tstamp)
-  {
-    u32 rtt_sample = 0, pkt_received = msg->pkt_received TO_US;
-    int remote_time, full_time;
-
-    /* processing time reported by peer */
-    remote_time = (n->last_tstamp - msg->tstamp_rcvd);
-    /* time since we sent the last timestamp - RTT including remote time */
-    full_time = (pkt_received - msg->tstamp);
-
-    /* sanity checks */
-    if (remote_time < 0 || full_time < 0 ||
-        remote_time US_ > BABEL_RTT_MAX_VALUE || full_time US_ > BABEL_RTT_MAX_VALUE)
-      goto out;
-
-    if (remote_time < full_time)
-      rtt_sample = full_time - remote_time;
-
-    if (n->srtt)
-    {
-      uint decay = n->ifa->cf->rtt_decay;
-
-      n->srtt = (decay * rtt_sample + (256 - decay) * n->srtt) / 256;
-    }
-    else
-      n->srtt = rtt_sample;
-
-    TRACE(D_EVENTS, "RTT sample for neighbour %I on %s: %u us (srtt %t ms)",
-          n->addr, ifa->ifname, rtt_sample, n->srtt * 1000);
-  }
-
-out:
   babel_update_cost(n);
 }
 
@@ -2062,7 +1989,7 @@ babel_reconfigure_ifaces(struct babel_proto *p, struct babel_config *cf)
 
   WALK_LIST(iface, iface_list)
   {
-    if (p->p.vrf_set && !if_in_vrf(iface, p->p.vrf))
+    if (p->p.vrf_set && p->p.vrf != iface->master)
       continue;
 
     if (!(iface->flags & IF_UP))
@@ -2103,82 +2030,82 @@ babel_reconfigure_ifaces(struct babel_proto *p, struct babel_config *cf)
  */
 
 static void
-babel_dump_source(struct dump_request *dreq, struct babel_source *s)
+babel_dump_source(struct babel_source *s)
 {
-  RDUMP("Source router_id %lR seqno %d metric %d expires %t\n",
+  debug("Source router_id %lR seqno %d metric %d expires %t\n",
 	s->router_id, s->seqno, s->metric,
 	s->expires ? s->expires - current_time() : 0);
 }
 
 static void
-babel_dump_route(struct dump_request *dreq, struct babel_route *r)
+babel_dump_route(struct babel_route *r)
 {
-  RDUMP("Route neigh %I if %s seqno %d metric %d/%d router_id %lR expires %t\n",
+  debug("Route neigh %I if %s seqno %d metric %d/%d router_id %lR expires %t\n",
 	r->neigh->addr, r->neigh->ifa->ifname, r->seqno, r->advert_metric, r->metric,
 	r->router_id, r->expires ? r->expires - current_time() : 0);
 }
 
 static void
-babel_dump_entry(struct dump_request *dreq, struct babel_entry *e)
+babel_dump_entry(struct babel_entry *e)
 {
   struct babel_source *s;
   struct babel_route *r;
 
-  RDUMP("Babel: Entry %N:\n", e->n.addr);
+  debug("Babel: Entry %N:\n", e->n.addr);
 
   WALK_LIST(s,e->sources)
-  { RDUMP(" "); babel_dump_source(dreq, s); }
+  { debug(" "); babel_dump_source(s); }
 
   WALK_LIST(r,e->routes)
   {
-    RDUMP(" ");
-    if (r == e->selected) RDUMP("*");
-    babel_dump_route(dreq, r);
+    debug(" ");
+    if (r == e->selected) debug("*");
+    babel_dump_route(r);
   }
 }
 
 static void
-babel_dump_neighbor(struct dump_request *dreq, struct babel_neighbor *n)
+babel_dump_neighbor(struct babel_neighbor *n)
 {
-  RDUMP("Neighbor %I txcost %d hello_map %x next seqno %d expires %t/%t\n",
+  debug("Neighbor %I txcost %d hello_map %x next seqno %d expires %t/%t\n",
 	n->addr, n->txcost, n->hello_map, n->next_hello_seqno,
 	n->hello_expiry ? n->hello_expiry - current_time() : 0,
         n->ihu_expiry ? n->ihu_expiry - current_time() : 0);
 }
 
 static void
-babel_dump_iface(struct dump_request *dreq, struct babel_iface *ifa)
+babel_dump_iface(struct babel_iface *ifa)
 {
   struct babel_neighbor *n;
 
-  RDUMP("Babel: Interface %s addr %I rxcost %d type %d hello seqno %d intervals %t %t",
+  debug("Babel: Interface %s addr %I rxcost %d type %d hello seqno %d intervals %t %t",
 	ifa->ifname, ifa->addr, ifa->cf->rxcost, ifa->cf->type, ifa->hello_seqno,
 	ifa->cf->hello_interval, ifa->cf->update_interval);
-  RDUMP(" next hop v4 %I next hop v6 %I\n", ifa->next_hop_ip4, ifa->next_hop_ip6);
+  debug(" next hop v4 %I next hop v6 %I\n", ifa->next_hop_ip4, ifa->next_hop_ip6);
 
   WALK_LIST(n, ifa->neigh_list)
-  { RDUMP(" "); babel_dump_neighbor(dreq, n); }
+  { debug(" "); babel_dump_neighbor(n); }
 }
 
 static void
-babel_dump(struct proto *P, struct dump_request *dreq)
+babel_dump(struct proto *P)
 {
   struct babel_proto *p = (struct babel_proto *) P;
   struct babel_iface *ifa;
 
-  RDUMP("Babel: router id %lR update seqno %d\n", p->router_id, p->update_seqno);
+  debug("Babel: router id %lR update seqno %d\n", p->router_id, p->update_seqno);
 
   WALK_LIST(ifa, p->interfaces)
-    babel_dump_iface(dreq, ifa);
+    babel_dump_iface(ifa);
 
   FIB_WALK(&p->ip4_rtable, struct babel_entry, e)
   {
-    babel_dump_entry(dreq, e);
+    babel_dump_entry(e);
   }
   FIB_WALK_END;
   FIB_WALK(&p->ip6_rtable, struct babel_entry, e)
   {
-    babel_dump_entry(dreq, e);
+    babel_dump_entry(e);
   }
   FIB_WALK_END;
 }
@@ -2272,8 +2199,8 @@ babel_show_neighbors(struct proto *P, const char *iff)
   }
 
   cli_msg(-1024, "%s:", p->p.name);
-  cli_msg(-1024, "%-25s %-10s %6s %6s %6s %7s %4s %9s",
-	  "IP address", "Interface", "Metric", "Routes", "Hellos", "Expires", "Auth", "RTT (ms)");
+  cli_msg(-1024, "%-25s %-10s %6s %6s %6s %7s %4s",
+	  "IP address", "Interface", "Metric", "Routes", "Hellos", "Expires", "Auth");
 
   WALK_LIST(ifa, p->interfaces)
   {
@@ -2288,10 +2215,9 @@ babel_show_neighbors(struct proto *P, const char *iff)
 
       uint hellos = u32_popcount(n->hello_map);
       btime timer = (n->hello_expiry ?: n->init_expiry) - current_time();
-      cli_msg(-1024, "%-25I %-10s %6u %6u %6u %7t %-4s %9t",
+      cli_msg(-1024, "%-25I %-10s %6u %6u %6u %7t %-4s",
 	      n->addr, ifa->iface->name, n->cost, rts, hellos, MAX(timer, 0),
-              n->auth_passed ? "Yes" : "No",
-              n->srtt * 1000);
+              n->auth_passed ? "Yes" : "No");
     }
   }
 }
