@@ -53,16 +53,10 @@
 #define BABEL_GARBAGE_INTERVAL		(300 S_)
 #define BABEL_RXCOST_WIRED		96
 #define BABEL_RXCOST_WIRELESS		256
-#define BABEL_RXCOST_RTT		96
 #define BABEL_INITIAL_HOP_COUNT		255
 #define BABEL_MAX_SEND_INTERVAL		5	/* Unused ? */
 #define BABEL_INITIAL_NEIGHBOR_TIMEOUT	(60 S_)
 
-#define BABEL_RTT_MAX_VALUE		(600 S_)
-#define BABEL_RTT_MIN			(10 MS_)
-#define BABEL_RTT_MAX			(120 MS_)
-#define BABEL_RTT_DECAY			42
-
 /* Max interval that will not overflow when carried as 16-bit centiseconds */
 #define BABEL_TIME_UNITS		10000	/* On-wire times are counted in centiseconds */
 #define BABEL_MIN_INTERVAL		(0x0001 * BABEL_TIME_UNITS)
@@ -102,8 +96,6 @@ enum babel_tlv_type {
 enum babel_subtlv_type {
   BABEL_SUBTLV_PAD1		= 0,
   BABEL_SUBTLV_PADN		= 1,
-  BABEL_SUBTLV_DIVERSITY	= 2, /* we don't support this */
-  BABEL_SUBTLV_TIMESTAMP	= 3,
 
   /* Mandatory subtlvs */
   BABEL_SUBTLV_SOURCE_PREFIX    = 128,
@@ -114,15 +106,9 @@ enum babel_iface_type {
   BABEL_IFACE_TYPE_UNDEF	= 0,
   BABEL_IFACE_TYPE_WIRED	= 1,
   BABEL_IFACE_TYPE_WIRELESS	= 2,
-  BABEL_IFACE_TYPE_TUNNEL	= 3,
   BABEL_IFACE_TYPE_MAX
 };
 
-enum babel_next_hop_prefer {
-  BABEL_NHP_NATIVE		= 0,
-  BABEL_NHP_IP6			= 1,
-};
-
 enum babel_ae_type {
   BABEL_AE_WILDCARD		= 0,
   BABEL_AE_IP4			= 1,
@@ -155,12 +141,6 @@ struct babel_iface_config {
   uint ihu_interval;			/* IHU interval, in us */
   uint update_interval;			/* Update interval, in us */
 
-  btime rtt_min;			/* rtt above which to start penalising metric */
-  btime rtt_max;			/* max rtt metric penalty applied above this */
-  u16 rtt_cost;			/* metric penalty to apply at rtt_max */
-  u16 rtt_decay;			/* decay of neighbour RTT (units of 1/256) */
-  u8  rtt_send;			/* whether to send timestamps on this interface */
-
   u16 rx_buffer;			/* RX buffer size, 0 for MTU */
   u16 tx_length;			/* TX packet length limit (including headers), 0 for MTU */
   int tx_tos;
@@ -168,7 +148,6 @@ struct babel_iface_config {
 
   ip_addr next_hop_ip4;
   ip_addr next_hop_ip6;
-  u8 next_hop_prefer;			/* Prefer native or IPv6 nexthop */
   u8 ext_next_hop;			/* Enable IPv4 via IPv6 */
 
   u8 auth_type;				/* Authentication type (BABEL_AUTH_*) */
@@ -250,10 +229,6 @@ struct babel_neighbor {
   u16 next_hello_seqno;
   uint last_hello_int;
 
-  u32 last_tstamp;
-  btime last_tstamp_rcvd;
-  btime srtt;
-
   u32 auth_pc_unicast;
   u32 auth_pc_multicast;
   u8 auth_passed;
@@ -351,8 +326,6 @@ struct babel_msg_hello {
   u16 seqno;
   uint interval;
   ip_addr sender;
-  u32 tstamp;
-  btime pkt_received;
 };
 
 struct babel_msg_ihu {
@@ -362,9 +335,6 @@ struct babel_msg_ihu {
   uint interval;
   ip_addr addr;
   ip_addr sender;
-  u32 tstamp;
-  u32 tstamp_rcvd;
-  btime pkt_received;
 };
 
 struct babel_msg_update {
