@@ -86,8 +86,9 @@
  * RFC 5065 - AS confederations for BGP
  * RFC 5082 - Generalized TTL Security Mechanism
  * RFC 5492 - Capabilities Advertisement with BGP
+ * RFC 5549 - Advertising IPv4 NLRI with an IPv6 Next Hop
+ * RFC 5575 - Dissemination of Flow Specification Rules
  * RFC 5668 - 4-Octet AS Specific BGP Extended Community
- * RFC 5925 - TCP Authentication Option
  * RFC 6286 - AS-Wide Unique BGP Identifier
  * RFC 6608 - Subcodes for BGP Finite State Machine Error
  * RFC 6793 - BGP Support for 4-Octet AS Numbers
@@ -97,17 +98,13 @@
  * RFC 7911 - Advertisement of Multiple Paths in BGP
  * RFC 7947 - Internet Exchange BGP Route Server
  * RFC 8092 - BGP Large Communities Attribute
+ * RFC 8203 - BGP Administrative Shutdown Communication
  * RFC 8212 - Default EBGP Route Propagation Behavior without Policies
  * RFC 8654 - Extended Message Support for BGP
- * RFC 8950 - Advertising IPv4 NLRI with an IPv6 Next Hop
- * RFC 8955 - Dissemination of Flow Specification Rules
- * RFC 8956 - Dissemination of Flow Specification Rules for IPv6
- * RFC 9003 - Extended BGP Administrative Shutdown Communication
  * RFC 9072 - Extended Optional Parameters Length for BGP OPEN Message
  * RFC 9117 - Revised Validation Procedure for BGP Flow Specifications
  * RFC 9234 - Route Leak Prevention and Detection Using Roles
- * RFC 9494 - Long-Lived Graceful Restart for BGP
- * RFC 9687 - Send Hold Timer
+ * draft-uttaro-idr-bgp-persistence-04
  * draft-walton-bgp-hostname-capability-02
  */
 
@@ -141,7 +138,6 @@ static void bgp_setup_sk(struct bgp_conn *conn, sock *s);
 static void bgp_send_open(struct bgp_conn *conn);
 static void bgp_update_bfd(struct bgp_proto *p, const struct bfd_options *bfd);
 
-static int bgp_disable_ao_keys(struct bgp_proto *p);
 static int bgp_incoming_connection(sock *sk, uint dummy UNUSED);
 static void bgp_listen_sock_err(sock *sk UNUSED, int err);
 
@@ -234,462 +230,10 @@ bgp_close(struct bgp_proto *p)
   mb_free(bs);
 }
 
-
-/*
- *	TCP-AO keys
- */
-
-static struct bgp_ao_key *
-bgp_new_ao_key(struct bgp_proto *p, struct ao_config *cf)
-{
-  struct bgp_ao_key *key = mb_allocz(p->p.pool, sizeof(struct bgp_ao_key));
-
-  key->key = cf->key;
-  add_tail(&p->ao.keys, &key->n);
-
-  return key;
-}
-
-static struct bgp_ao_key *
-bgp_find_ao_key_(list *l, int send_id, int recv_id)
-{
-  WALK_LIST_(struct bgp_ao_key, key, *l)
-    if ((key->key.send_id == send_id) && (key->key.recv_id == recv_id))
-      return key;
-
-  return NULL;
-}
-
-static inline struct bgp_ao_key * UNUSED
-bgp_find_ao_key(struct bgp_proto *p, int send_id, int recv_id)
-{ return bgp_find_ao_key_(&p->ao.keys, send_id, recv_id); }
-
-static int
-bgp_same_ao_key(struct ao_key *a, struct ao_key *b)
-{
-  return
-    (a->send_id == b->send_id) &&
-    (a->recv_id == b->recv_id) &&
-    (a->algorithm == b->algorithm) &&
-    (a->keylen == b->keylen) &&
-    !memcmp(a->key, b->key, a->keylen);
-}
-
 static inline int
-bgp_sk_add_ao_key(struct bgp_proto *p, sock *sk, struct bgp_ao_key *key)
-{
-  ip_addr prefix = p->cf->remote_ip;
-  int pxlen = -1;
-
-  int rv = sk_add_ao_key(sk, prefix, pxlen, p->cf->iface, &key->key, false, false);
-  if (rv < 0)
-  {
-    sk_log_error(sk, p->p.name);
-    log(L_ERR "%s: Cannot add TCP-AO key %d/%d to BGP %s socket",
-	p->p.name, key->key.send_id, key->key.recv_id,
-	((sk == p->sock->sk) ? "listening" : "session"));
-  }
-
-  return rv;
-}
-
-static int
-bgp_enable_ao_key(struct bgp_proto *p, struct bgp_ao_key *key)
-{
-  ASSERT(!key->active);
-
-  BGP_TRACE(D_EVENTS, "Adding TCP-AO key %d/%d", key->key.send_id, key->key.recv_id);
-
-  /* Handle listening socket */
-  if (bgp_sk_add_ao_key(p, p->sock->sk, key) < 0)
-  {
-    key->failed = 1;
-    return -1;
-  }
-
-  key->active = 1;
-
-  /* Handle incoming socket */
-  if (p->incoming_conn.sk)
-    if (bgp_sk_add_ao_key(p, p->incoming_conn.sk, key) < 0)
-      return -1;
-
-  /* Handle outgoing socket */
-  if (p->outgoing_conn.sk)
-    if (bgp_sk_add_ao_key(p, p->outgoing_conn.sk, key) < 0)
-      return -1;
-
-  return 0;
-}
-
-struct bgp_active_keys {
-  int in_current, in_rnext;
-  int out_current, out_rnext;
-  struct bgp_ao_key *backup;
-};
-
-static int
-bgp_sk_delete_ao_key(struct bgp_proto *p, sock *sk, struct bgp_ao_key *key,
-		     struct bgp_ao_key *backup, int current_key_id, int rnext_key_id)
-{
-  struct ao_key *set_current = NULL, *set_rnext = NULL;
-
-  if ((key->key.send_id == current_key_id) && backup)
-  {
-    log(L_WARN "%s: Deleting TCP-AO Current key %d/%d, setting Current key to %d/%d",
-	p->p.name, key->key.send_id, key->key.recv_id, backup->key.send_id, backup->key.recv_id);
-
-    set_current = &backup->key;
-  }
-
-  if ((key->key.recv_id == rnext_key_id) && backup)
-  {
-    log(L_WARN "%s: Deleting TCP-AO RNext key %d/%d, setting RNext key to %d/%d",
-	p->p.name, key->key.send_id, key->key.recv_id, backup->key.send_id, backup->key.recv_id);
-
-    set_rnext = &backup->key;
-  }
-
-  ip_addr prefix = p->cf->remote_ip;
-  int pxlen = -1;
-
-  int rv = sk_delete_ao_key(sk, prefix, pxlen, p->cf->iface, &key->key, set_current, set_rnext);
-  if (rv < 0)
-  {
-    sk_log_error(sk, p->p.name);
-    log(L_ERR "%s: Cannot delete TCP-AO key %d/%d from BGP %s socket",
-	p->p.name, key->key.send_id, key->key.recv_id,
-	((sk == p->sock->sk) ? "listening" : "session"));
-  }
-
-  return rv;
-}
-
-static int
-bgp_disable_ao_key(struct bgp_proto *p, struct bgp_ao_key *key, struct bgp_active_keys *info)
-{
-  ASSERT(key->active);
-
-  BGP_TRACE(D_EVENTS, "Deleting TCP-AO key %d/%d", key->key.send_id, key->key.recv_id);
-
-  /* Handle listening socket */
-  if (bgp_sk_delete_ao_key(p, p->sock->sk, key, NULL, -1, -1) < 0)
-    return -1;
-
-  key->active = 0;
-
-  /* Handle incoming socket */
-  if (p->incoming_conn.sk && info)
-    if (bgp_sk_delete_ao_key(p, p->incoming_conn.sk, key, info->backup, info->in_current, info->in_rnext) < 0)
-      return -1;
-
-  /* Handle outgoing socket */
-  if (p->outgoing_conn.sk && info)
-    if (bgp_sk_delete_ao_key(p, p->outgoing_conn.sk, key, info->backup, info->out_current, info->out_rnext) < 0)
-      return -1;
-
-  return 0;
-}
-
-static int
-bgp_remove_ao_key(struct bgp_proto *p, struct bgp_ao_key *key, struct bgp_active_keys *info)
-{
-  ASSERT(key != p->ao.best_key);
-
-  if (key->active)
-    if (bgp_disable_ao_key(p, key, info) < 0)
-      return -1;
-
-  rem_node(&key->n);
-  mb_free(key);
-  return 0;
-}
-
-
-static struct bgp_ao_key *
-bgp_select_best_ao_key(struct bgp_proto *p)
-{
-  struct bgp_ao_key *best = NULL;
-
-  WALK_LIST_(struct bgp_ao_key, key, p->ao.keys)
-  {
-    if (!key->active)
-      continue;
-
-    /* Never select deprecated keys */
-    if (key->key.preference < 0)
-      continue;
-
-    if (!best || (best->key.preference < key->key.preference))
-      best = key;
-  }
-
-  return best;
-}
-
-static int
-bgp_sk_set_rnext_ao_key(struct bgp_proto *p, sock *sk, struct bgp_ao_key *key)
-{
-  int rv = sk_set_rnext_ao_key(sk, &key->key);
-  if (rv < 0)
-  {
-    sk_log_error(sk, p->p.name);
-    log(L_ERR "%s: Cannot set TCP-AO key %d/%d as RNext key",
-	p->p.name, key->key.send_id, key->key.recv_id);
-  }
-
-  return rv;
-}
-
-static int
-bgp_update_rnext_ao_key(struct bgp_proto *p)
-{
-  struct bgp_ao_key *best = bgp_select_best_ao_key(p);
-
-  if (!best)
-  {
-    log(L_ERR "%s: No usable TCP-AO key", p->p.name);
-    return -1;
-  }
-
-  if (best == p->ao.best_key)
-    return 0;
-
-  BGP_TRACE(D_EVENTS, "Setting TCP-AO key %d/%d as RNext key", best->key.send_id, best->key.recv_id);
-
-  p->ao.best_key = best;
-
-  /* Handle incoming socket */
-  if (p->incoming_conn.sk)
-    if (bgp_sk_set_rnext_ao_key(p, p->incoming_conn.sk, best) < 0)
-      return -1;
-
-  /* Handle outgoing socket */
-  if (p->outgoing_conn.sk)
-    if (bgp_sk_set_rnext_ao_key(p, p->outgoing_conn.sk, best) < 0)
-      return -1;
-
-  /* Schedule Keepalive to trigger RNext ID exchange */
-  if (p->conn)
-    bgp_schedule_packet(p->conn, NULL, PKT_KEEPALIVE);
-
-  /* RFC 4271 4.4 says that Keepalive messages MUST NOT be sent more frequently
-     than one per second, but since key change is rare, this is harmless. */
-
-  return 0;
-}
-
-
-/**
- * bgp_enable_ao_keys - Enable TCP-AO keys
- * @p: BGP instance
- *
- * Enable all TCP-AO keys for the listening socket. We accept if some fail in
- * non-fatal way (e.g. kernel does not support specific algorithm), but there
- * must be at least one usable (non-deprecated) active key. In case of failure,
- * we remove all keys, so there is no lasting effect on the listening socket.
- * Returns: 0 for okay, -1 for failure.
- */
-static int
-bgp_enable_ao_keys(struct bgp_proto *p)
-{
-  ASSERT(!p->incoming_conn.sk && !p->outgoing_conn.sk);
-
-  WALK_LIST_(struct bgp_ao_key, key, p->ao.keys)
-    if (bgp_enable_ao_key(p, key) < 0)
-      goto fail;
-
-  p->ao.best_key = bgp_select_best_ao_key(p);
-
-  if (!p->ao.best_key)
-  {
-    log(L_ERR "%s: No usable TCP-AO key", p->p.name);
-    goto fail;
-  }
-
-  return 0;
-
-fail:
-  bgp_disable_ao_keys(p);
-  return -1;
-}
-
-/**
- * bgp_disable_ao_keys - Disable TCP-AO keys
- * @p: BGP instance
- *
- * Disable all TCP-AO keys for the listening socket. We assume there are no
- * active connection, so no issue with removal of the current key. Errors are
- * ignored.
- */
-static int
-bgp_disable_ao_keys(struct bgp_proto *p)
-{
-  ASSERT(!p->incoming_conn.sk && !p->outgoing_conn.sk);
-
-  WALK_LIST_(struct bgp_ao_key, key, p->ao.keys)
-    if (key->active)
-      bgp_disable_ao_key(p, key, NULL);
-
-  return 0;
-}
-
-static int
-bgp_reconfigure_ao_keys(struct bgp_proto *p, const struct bgp_config *cf)
-{
-  /* TCP-AO not used */
-  if (EMPTY_LIST(p->ao.keys) && !cf->ao_keys)
-    return 1;
-
-  /* Cannot enable/disable TCP-AO */
-  if (EMPTY_LIST(p->ao.keys) || !cf->ao_keys)
-    return 0;
-
-  /* Too early, TCP-AO not yet enabled */
-  if (p->start_state == BSS_PREPARE)
-    return 0;
-
-  /* Move existing keys to temporary list */
-  list old_keys;
-  init_list(&old_keys);
-  add_tail_list(&old_keys, &p->ao.keys);
-  init_list(&p->ao.keys);
-
-  /* Clean up the best key */
-  struct bgp_ao_key *old_best = p->ao.best_key;
-  p->ao.best_key = NULL;
-
-  /* Prepare new set of keys */
-  for (struct ao_config *key_cf = cf->ao_keys; key_cf; key_cf = key_cf->next)
-  {
-    struct bgp_ao_key *key = bgp_find_ao_key_(&old_keys, key_cf->key.send_id, key_cf->key.recv_id);
-
-    if (key && bgp_same_ao_key(&key->key, &key_cf->key))
-    {
-      /* Update key ptr and preference */
-      key->key = key_cf->key;
-
-      rem_node(&key->n);
-      add_tail(&p->ao.keys, &key->n);
-
-      if (key == old_best)
-	p->ao.best_key = key;
-
-      continue;
-    }
-
-    bgp_new_ao_key(p, key_cf);
-  }
-
-  /* Remove old keys */
-  if (!EMPTY_LIST(old_keys))
-  {
-    struct bgp_active_keys info = { -1, -1, -1, -1, NULL};
-
-    /* Find current/rnext keys on incoming connection */
-    if (p->incoming_conn.sk)
-      if (sk_get_active_ao_keys(p->incoming_conn.sk, &info.in_current, &info.in_rnext) < -1)
-	sk_log_error(p->incoming_conn.sk, p->p.name);
-
-    /* Find current/rnext keys on outgoing connection */
-    if (p->outgoing_conn.sk)
-      if (sk_get_active_ao_keys(p->outgoing_conn.sk, &info.out_current, &info.out_rnext) < -1)
-	sk_log_error(p->outgoing_conn.sk, p->p.name);
-
-    /*
-     * Select backup key in case of removal of current/rnext key.
-     *
-     * It is possible that we cannot select an intermediate best key (e.g. when
-     * the reconfiguration deprecates the old best key and adds the new one).
-     * That is not necessary bad, we may not even need the backup key anyways.
-     * In this case we use the old best key (ao.best_key) instead even if it may
-     * be deprecated (but not removed).
-     *
-     * If neither one is available, that means we are going to remove rnext key
-     * and we have no intermediate best key to switch to, therefore we fail
-     * later during bgp_remove_ao_key().
-     */
-    info.backup = bgp_select_best_ao_key(p) ?: p->ao.best_key;
-    if (!info.backup)
-      log(L_WARN "%s: No usable backup key", p->p.name);
-
-    struct bgp_ao_key *key, *key2;
-    WALK_LIST_DELSAFE(key, key2, old_keys)
-      bgp_remove_ao_key(p, key, &info);
-
-    /* If some key removals failed */
-    if (!EMPTY_LIST(old_keys))
-      return 0;
-  }
-
-  /* Enable new keys */
-  WALK_LIST_(struct bgp_ao_key, key, p->ao.keys)
-    if (!key->active && !key->failed)
-      if (bgp_enable_ao_key(p, key) < 0)
-	return 0;
-
-  /* Update RNext key */
-  if (bgp_update_rnext_ao_key(p) < 0)
-    return 0;
-
-  return 1;
-}
-
-/**
- * bgp_list_ao_keys - List active TCP-AO keys
- * @p: BGP instance
- * @ao_keys: Returns array of keys
- * @ao_keys_num: Returns number of keys
- *
- * Returns an array of pointers to active TCP-AO keys, for usage with socket
- * functions. The best key is at the first position. The array is allocated from
- * the temporary linpool. If there are no keys (or just no best key), the error
- * is logged and the function fails. Returns: 0 for success, -1 for failure.
- */
-static int
-bgp_list_ao_keys(struct bgp_proto *p, const struct ao_key ***ao_keys, int *ao_keys_num)
-{
-  int num = 0;
-  WALK_LIST_(const struct bgp_ao_key, key, p->ao.keys)
-    if (key->active)
-      num++;
-
-  const struct bgp_ao_key *best = p->ao.best_key;
-
-  if (!num || !best)
-  {
-    log(L_ERR "%s: No usable TCP-AO key", p->p.name);
-    return -1;
-  }
-
-  const struct ao_key **keys = tmp_alloc(num * sizeof(const struct ao_key *));
-  int i = 0;
-
-  keys[i++] = &best->key;
-  WALK_LIST_(const struct bgp_ao_key, key, p->ao.keys)
-    if (key->active && (key != best) && (i < num))
-      keys[i++] = &key->key;
-
-  *ao_keys = keys;
-  *ao_keys_num = i;
-  return 0;
-}
-
-
-
-
-static int
 bgp_setup_auth(struct bgp_proto *p, int enable)
 {
-  if (p->cf->auth_type == BGP_AUTH_AO)
-  {
-    if (enable)
-      return bgp_enable_ao_keys(p);
-    else
-      return bgp_disable_ao_keys(p);
-  }
-
-  if (p->cf->auth_type == BGP_AUTH_MD5)
+  if (p->cf->password)
   {
     ip_addr prefix = p->cf->remote_ip;
     int pxlen = -1;
@@ -709,8 +253,8 @@ bgp_setup_auth(struct bgp_proto *p, int enable)
 
     return rv;
   }
-
-  return 0;
+  else
+    return 0;
 }
 
 static inline struct bgp_channel *
@@ -759,7 +303,7 @@ bgp_initiate(struct bgp_proto *p)
   { err_val = BEM_NO_SOCKET; goto err1; }
 
   if (bgp_setup_auth(p, 1) < 0)
-  { err_val = BEM_INVALID_AUTH; goto err2; }
+  { err_val = BEM_INVALID_MD5; goto err2; }
 
   if (p->cf->bfd)
     bgp_update_bfd(p, p->cf->bfd);
@@ -831,27 +375,15 @@ bgp_close_conn(struct bgp_conn *conn)
   conn->keepalive_timer = NULL;
   rfree(conn->hold_timer);
   conn->hold_timer = NULL;
-  rfree(conn->send_hold_timer);
-  conn->send_hold_timer = NULL;
   rfree(conn->tx_ev);
   conn->tx_ev = NULL;
   rfree(conn->sk);
   conn->sk = NULL;
 
-  mb_free(conn->local_open_msg);
-  conn->local_open_msg = NULL;
-  mb_free(conn->remote_open_msg);
-  conn->remote_open_msg = NULL;
-  conn->local_open_length = 0;
-  conn->remote_open_length = 0;
-
   mb_free(conn->local_caps);
   conn->local_caps = NULL;
   mb_free(conn->remote_caps);
   conn->remote_caps = NULL;
-
-  conn->notify_data = NULL;
-  conn->notify_size = 0;
 }
 
 
@@ -965,8 +497,8 @@ bgp_spawn(struct bgp_proto *pp, ip_addr remote_ip)
   /* This is hack, we would like to share config, but we need to copy it now */
   new_config = config;
   cfg_mem = config->mem;
-  config->current_scope = config->root_scope;
-  sym = cf_default_name(config, fmt, &(pp->dynamic_name_counter));
+  conf_this_scope = config->root_scope;
+  sym = cf_default_name(fmt, &(pp->dynamic_name_counter));
   proto_clone_config(sym, pp->p.cf);
   new_config = NULL;
   cfg_mem = NULL;
@@ -980,8 +512,6 @@ bgp_spawn(struct bgp_proto *pp, ip_addr remote_ip)
 void
 bgp_stop(struct bgp_proto *p, int subcode, byte *data, uint len)
 {
-  proto_shutdown_mpls_map(&p->p, 1);
-
   proto_notify_state(&p->p, PS_STOP);
   bgp_graceful_close_conn(&p->outgoing_conn, subcode, data, len);
   bgp_graceful_close_conn(&p->incoming_conn, subcode, data, len);
@@ -1004,6 +534,9 @@ bgp_conn_enter_openconfirm_state(struct bgp_conn *conn)
   bgp_conn_set_state(conn, BS_OPENCONFIRM);
 }
 
+static const struct bgp_af_caps dummy_af_caps = { };
+static const struct bgp_af_caps basic_af_caps = { .ready = 1 };
+
 void
 bgp_conn_enter_established_state(struct bgp_conn *conn)
 {
@@ -1059,6 +592,20 @@ bgp_conn_enter_established_state(struct bgp_conn *conn)
     const struct bgp_af_caps *loc = bgp_find_af_caps(local, c->afi);
     const struct bgp_af_caps *rem = bgp_find_af_caps(peer,  c->afi);
 
+    /* Use default if capabilities were not announced */
+    if (!local->length && (c->afi == BGP_AF_IPV4))
+      loc = &basic_af_caps;
+
+    if (!peer->length && (c->afi == BGP_AF_IPV4))
+      rem = &basic_af_caps;
+
+    /* Ignore AFIs that were not announced in multiprotocol capability */
+    if (!loc || !loc->ready)
+      loc = &dummy_af_caps;
+
+    if (!rem || !rem->ready)
+      rem = &dummy_af_caps;
+
     int active = loc->ready && rem->ready;
     c->c.disabled = !active;
     c->c.reloadable = p->route_refresh || c->cf->import_table;
@@ -1077,8 +624,7 @@ bgp_conn_enter_established_state(struct bgp_conn *conn)
     p->llgr_ready = p->llgr_ready || llgr_ready;
 
     /* Remember last LLGR stale time */
-    c->stale_time = local->llgr_aware ?
-      CLAMP(rem->llgr_time, c->cf->min_llgr_time, c->cf->max_llgr_time) : 0;
+    c->stale_time = local->llgr_aware ? rem->llgr_time : 0;
 
     /* Channels not able to recover gracefully */
     if (p->p.gr_recovery && (!active || !peer_gr_ready))
@@ -1132,23 +678,14 @@ bgp_conn_enter_established_state(struct bgp_conn *conn)
     p->channel_map[c->index] = c;
   }
 
-  /* Breaking rx_hook for simulating receive problem */
-  if (p->cf->disable_rx)
-  {
-    conn->sk->rx_hook = NULL;
-    tm_stop(conn->hold_timer);
-  }
-
   /* proto_notify_state() will likely call bgp_feed_begin(), setting c->feed_state */
 
   bgp_conn_set_state(conn, BS_ESTABLISHED);
   proto_notify_state(&p->p, PS_UP);
-  bmp_peer_up(p, conn->local_open_msg, conn->local_open_length,
-	      conn->remote_open_msg, conn->remote_open_length);
 }
 
 static void
-bgp_conn_leave_established_state(struct bgp_conn *conn, struct bgp_proto *p)
+bgp_conn_leave_established_state(struct bgp_proto *p)
 {
   BGP_TRACE(D_EVENTS, "BGP session closed");
   p->last_established = current_time();
@@ -1156,10 +693,6 @@ bgp_conn_leave_established_state(struct bgp_conn *conn, struct bgp_proto *p)
 
   if (p->p.proto_state == PS_UP)
     bgp_stop(p, 0, NULL, 0);
-
-  bmp_peer_down(p, p->last_error_class,
-		conn->notify_code, conn->notify_subcode,
-		conn->notify_data, conn->notify_size);
 }
 
 void
@@ -1176,7 +709,7 @@ bgp_conn_enter_close_state(struct bgp_conn *conn)
   bgp_start_timer(conn->hold_timer, 10);
 
   if (os == BS_ESTABLISHED)
-    bgp_conn_leave_established_state(conn, p);
+    bgp_conn_leave_established_state(p);
 }
 
 void
@@ -1190,7 +723,7 @@ bgp_conn_enter_idle_state(struct bgp_conn *conn)
   ev_schedule(p->event);
 
   if (os == BS_ESTABLISHED)
-    bgp_conn_leave_established_state(conn, p);
+    bgp_conn_leave_established_state(p);
 }
 
 /**
@@ -1261,11 +794,8 @@ bgp_handle_graceful_restart(struct bgp_proto *p)
   /* p->gr_ready -> at least one active channel is c->gr_ready */
   ASSERT(p->gr_active_num > 0);
 
-  uint gr_time = CLAMP(p->conn->remote_caps->gr_time,
-		       p->cf->min_gr_time, p->cf->max_gr_time);
-
   proto_notify_state(&p->p, PS_START);
-  tm_start(p->gr_timer, gr_time S);
+  tm_start(p->gr_timer, p->conn->remote_caps->gr_time S);
 }
 
 /**
@@ -1337,7 +867,10 @@ bgp_graceful_restart_timeout(timer *t)
     }
   }
   else
+  {
     bgp_stop(p, 0, NULL, 0);
+    bmp_peer_down(p, BE_NONE, NULL, 0);
+  }
 }
 
 static void
@@ -1461,7 +994,10 @@ bgp_sock_err(sock *sk, int err)
   if (err)
     BGP_TRACE(D_EVENTS, "Connection lost (%M)", err);
   else
+  {
     BGP_TRACE(D_EVENTS, "Connection closed");
+    bmp_peer_down(p, BE_SOCKET, NULL, 0);
+  }
 
   if ((conn->state == BS_ESTABLISHED) && p->gr_ready)
     bgp_handle_graceful_restart(p);
@@ -1513,28 +1049,6 @@ bgp_keepalive_timeout(timer *t)
     ev_run(conn->tx_ev);
 }
 
-void
-bgp_send_hold_timeout(timer *t)
-{
-  struct bgp_conn *conn = t->data;
-  struct bgp_proto *p = conn->bgp;
-
-  DBG("BGP: Send hold timeout\n");
-
-  if (conn->state == BS_CLOSE)
-    return;
-
-  uint code = 8;
-  uint subcode = 0;
-
-  /* Like bgp_error() but without NOTIFICATION */
-  bgp_log_error(p, BE_BGP_TX, "Error", code, subcode, NULL, 0);
-  bgp_store_error(p, conn, BE_BGP_TX, (code << 16) | subcode);
-  bgp_conn_enter_idle_state(conn);
-  bgp_update_startup_delay(p);
-  bgp_stop(p, 0, NULL, 0);
-}
-
 static void
 bgp_setup_conn(struct bgp_proto *p, struct bgp_conn *conn)
 {
@@ -1549,7 +1063,6 @@ bgp_setup_conn(struct bgp_proto *p, struct bgp_conn *conn)
   conn->connect_timer	= tm_new_init(p->p.pool, bgp_connect_timeout,	 conn, 0, 0);
   conn->hold_timer 	= tm_new_init(p->p.pool, bgp_hold_timeout,	 conn, 0, 0);
   conn->keepalive_timer	= tm_new_init(p->p.pool, bgp_keepalive_timeout, conn, 0, 0);
-  conn->send_hold_timer = tm_new_init(p->p.pool, bgp_send_hold_timeout, conn, 0, 0);
 
   conn->tx_ev = ev_new_init(p->p.pool, bgp_kick_tx, conn);
 }
@@ -1587,7 +1100,7 @@ static void
 bgp_connect(struct bgp_proto *p)	/* Enter Connect state and start establishing connection */
 {
   struct bgp_conn *conn = &p->outgoing_conn;
-  int hops = p->cf->multihop ?: 1;
+  int hops = p->cf->multihop ? : 1;
 
   DBG("BGP: Connecting\n");
   sock *s = sk_new(p->p.pool);
@@ -1601,6 +1114,7 @@ bgp_connect(struct bgp_proto *p)	/* Enter Connect state and start establishing c
   s->rbsize = p->cf->enable_extended_messages ? BGP_RX_BUFFER_EXT_SIZE : BGP_RX_BUFFER_SIZE;
   s->tbsize = p->cf->enable_extended_messages ? BGP_TX_BUFFER_EXT_SIZE : BGP_TX_BUFFER_SIZE;
   s->tos = IP_PREC_INTERNET_CONTROL;
+  s->password = p->cf->password;
   s->tx_hook = bgp_connected;
   s->flags = p->cf->free_bind ? SKF_FREEBIND : 0;
   BGP_TRACE(D_EVENTS, "Connecting to %I%J from local address %I%J",
@@ -1610,13 +1124,6 @@ bgp_connect(struct bgp_proto *p)	/* Enter Connect state and start establishing c
   bgp_setup_sk(conn, s);
   bgp_conn_set_state(conn, BS_CONNECT);
 
-  if (p->cf->auth_type == BGP_AUTH_MD5)
-    s->password = p->cf->password;
-
-  if (p->cf->auth_type == BGP_AUTH_AO)
-    if (bgp_list_ao_keys(p, &s->ao_keys_init, &s->ao_keys_num) < 0)
-      goto err2;
-
   if (sk_open(s) < 0)
     goto err;
 
@@ -1625,16 +1132,12 @@ bgp_connect(struct bgp_proto *p)	/* Enter Connect state and start establishing c
     if (sk_set_min_ttl(s, 256 - hops) < 0)
       goto err;
 
-  s->ao_keys_num = 0;
-  s->ao_keys_init = NULL;
-
   DBG("BGP: Waiting for connect success\n");
   bgp_start_timer(conn->connect_timer, p->cf->connect_retry_time);
   return;
 
 err:
   sk_log_error(s, p->p.name);
-err2:
   bgp_sock_err(s, 0);
   return;
 }
@@ -1702,20 +1205,6 @@ bgp_incoming_connection(sock *sk, uint dummy UNUSED)
     return 0;
   }
 
-  if ((p->p.proto_state != PS_DOWN) && !EMPTY_LIST(p->ao.keys))
-  {
-    int current = -1, rnext = -1;
-    sk_get_active_ao_keys(sk, &current, &rnext);
-
-    if (current < 0)
-    {
-      log(L_WARN "%s: Connection from address %I%J (port %d) has no TCP-AO key",
-          p->p.name, sk->daddr, ipa_is_link_local(sk->daddr) ? sk->iface : NULL, sk->dport);
-      rfree(sk);
-      return 0;
-    }
-  }
-
   /*
    * BIRD should keep multiple incoming connections in OpenSent state (for
    * details RFC 4271 8.2.1 par 3), but it keeps just one. Duplicate incoming
@@ -1748,7 +1237,7 @@ bgp_incoming_connection(sock *sk, uint dummy UNUSED)
     return 0;
   }
 
-  hops = p->cf->multihop ?: 1;
+  hops = p->cf->multihop ? : 1;
 
   if (sk_set_ttl(sk, p->cf->ttl_security ? 255 : hops) < 0)
     goto err;
@@ -1757,21 +1246,6 @@ bgp_incoming_connection(sock *sk, uint dummy UNUSED)
     if (sk_set_min_ttl(sk, 256 - hops) < 0)
       goto err;
 
-  if (!EMPTY_LIST(p->ao.keys))
-  {
-    const struct ao_key **ao_keys;
-    int ao_keys_num;
-
-    if (bgp_list_ao_keys(p, &ao_keys, &ao_keys_num) < 0)
-      goto err2;
-
-    if (sk_check_ao_keys(sk, ao_keys, ao_keys_num, p->p.name) < 0)
-      goto err2;
-
-    if (sk_set_rnext_ao_key(sk, ao_keys[0]) < 0)
-      goto err;
-  }
-
   if (p->cf->enable_extended_messages)
   {
     sk->rbsize = BGP_RX_BUFFER_EXT_SIZE;
@@ -1796,7 +1270,6 @@ bgp_incoming_connection(sock *sk, uint dummy UNUSED)
 
 err:
   sk_log_error(sk, p->p.name);
-err2:
   log(L_ERR "%s: Incoming connection aborted", p->p.name);
   rfree(sk);
   return 0;
@@ -1849,6 +1322,7 @@ bgp_neigh_notify(neighbor *n)
       bgp_store_error(p, NULL, BE_MISC, BEM_NEIGHBOR_LOST);
       /* Perhaps also run bgp_update_startup_delay(p)? */
       bgp_stop(p, 0, NULL, 0);
+      bmp_peer_down(p, BE_MISC, NULL, 0);
     }
   }
   else if (p->cf->check_link && !(n->iface->flags & IF_LINK_UP))
@@ -1860,6 +1334,7 @@ bgp_neigh_notify(neighbor *n)
       if (ps == PS_UP)
 	bgp_update_startup_delay(p);
       bgp_stop(p, 0, NULL, 0);
+      bmp_peer_down(p, BE_MISC, NULL, 0);
     }
   }
   else
@@ -1901,6 +1376,7 @@ bgp_bfd_notify(struct bfd_request *req)
       if (ps == PS_UP)
 	bgp_update_startup_delay(p);
       bgp_stop(p, 0, NULL, 0);
+      bmp_peer_down(p, BE_MISC, NULL, 0);
     }
   }
 }
@@ -1929,16 +1405,6 @@ bgp_reload_routes(struct channel *C)
   struct bgp_proto *p = (void *) C->proto;
   struct bgp_channel *c = (void *) C;
 
-  /* For MPLS channel, reload all MPLS-aware channels */
-  if (C == p->p.mpls_channel)
-  {
-    BGP_WALK_CHANNELS(p, c)
-      if ((c->desc->mpls) && (p->route_refresh || c->c.in_table))
-	bgp_reload_routes(&c->c);
-
-    return;
-  }
-
   /* Ignore non-BGP channels */
   if (C->channel != &channel_bgp)
     return;
@@ -2093,8 +1559,6 @@ bgp_start(struct proto *P)
   p->startup_timer = tm_new_init(p->p.pool, bgp_startup_timeout, p, 0, 0);
   p->gr_timer = tm_new_init(p->p.pool, bgp_graceful_restart_timeout, p, 0, 0);
 
-  p->hostname = proto_get_hostname(P->cf);
-
   p->local_id = proto_get_router_id(P->cf);
   if (p->rr_client)
     p->rr_cluster_id = p->cf->rr_cluster_id ? p->cf->rr_cluster_id : p->local_id;
@@ -2102,14 +1566,6 @@ bgp_start(struct proto *P)
   p->remote_id = 0;
   p->link_addr = IPA_NONE;
 
-  /* Initialize TCP-AO keys */
-  init_list(&p->ao.keys);
-  if (cf->auth_type == BGP_AUTH_AO)
-    for (struct ao_config *key_cf = cf->ao_keys; key_cf; key_cf = key_cf->next)
-      bgp_new_ao_key(p, key_cf);
-
-  proto_setup_mpls_map(P, RTS_BGP, 1);
-
   /* Lock all channels when in GR recovery mode */
   if (p->p.gr_recovery && p->cf->gr_mode)
   {
@@ -2125,7 +1581,6 @@ bgp_start(struct proto *P)
   struct object_lock *lock;
   lock = p->lock = olock_new(P->pool);
   lock->addr = p->remote_ip;
-  lock->addr_local = p->cf->local_ip;
   lock->port = p->cf->remote_port;
   lock->iface = p->cf->iface;
   lock->vrf = p->cf->iface ? NULL : p->p.vrf;
@@ -2212,7 +1667,7 @@ bgp_shutdown(struct proto *P)
   bgp_store_error(p, NULL, BE_MAN_DOWN, 0);
   p->startup_delay = 0;
 
-  /* RFC 9003 - shutdown communication */
+  /* RFC 8203 - shutdown communication */
   if (message)
   {
     uint msg_len = strlen(message);
@@ -2250,6 +1705,10 @@ bgp_init(struct proto_config *CF)
   P->rte_modify = bgp_rte_modify_stale;
   P->rte_igp_metric = bgp_rte_igp_metric;
 
+#ifdef CONFIG_BMP
+  P->rte_update_in_notify = bgp_rte_update_in_notify;
+#endif
+
   p->cf = cf;
   p->is_internal = (cf->local_as == cf->remote_as);
   p->is_interior = p->is_internal || cf->confederation_member;
@@ -2267,14 +1726,11 @@ bgp_init(struct proto_config *CF)
   if (cf->c.parent)
     cf->remote_ip = IPA_NONE;
 
-  /* Add all BGP channels */
+  /* Add all channels */
   struct bgp_channel_config *cc;
   BGP_CF_WALK_CHANNELS(cf, cc)
     proto_add_channel(P, &cc->c);
 
-  /* Add MPLS channel */
-  proto_configure_channel(P, &P->mpls_channel, proto_cf_mpls_channel(CF));
-
   return P;
 }
 
@@ -2541,21 +1997,6 @@ bgp_postconfig(struct proto_config *CF)
   if (interior && (cf->local_role != BGP_ROLE_UNDEFINED))
     log(L_WARN "BGP roles are not recommended to be used within AS confederations");
 
-  if (cf->require_enhanced_refresh && !(cf->enable_refresh && cf->enable_enhanced_refresh))
-    cf_warn("Enhanced refresh required but disabled");
-
-  if (cf->require_as4 && !cf->enable_as4)
-    cf_warn("AS4 support required but disabled");
-
-  if (cf->require_extended_messages && !cf->enable_extended_messages)
-    cf_warn("Extended messages required but not enabled");
-
-  if (cf->require_gr && !cf->gr_mode)
-    cf_warn("Graceful restart required but not enabled");
-
-  if (cf->require_llgr && !cf->llgr_mode)
-    cf_warn("Long-lived graceful restart required but not enabled");
-
   if (cf->require_roles && (cf->local_role == BGP_ROLE_UNDEFINED))
     cf_error("Local role must be set if roles are required");
 
@@ -2596,29 +2037,6 @@ bgp_postconfig(struct proto_config *CF)
     cf_error("Min keepalive time (%u) exceeds keepalive time (%u)",
 	     cf->min_keepalive_time, keepalive_time);
 
-  if (cf->min_gr_time > cf->max_gr_time)
-    cf_error("Min graceful restart time (%u) exceeds max graceful restart time (%u)",
-	     cf->min_gr_time, cf->max_gr_time);
-
-  if (cf->min_llgr_time > cf->max_llgr_time)
-    cf_error("Min long-lived stale time (%u) exceeds max long-lived stale time (%u)",
-	     cf->min_llgr_time, cf->max_llgr_time);
-
-  /* Legacy case: password option without authentication option */
-  if ((cf->auth_type == BGP_AUTH_NONE) && cf->password && !cf->ao_keys)
-  {
-    cf_warn("Missing authentication option, assuming MD5");
-    cf->auth_type = BGP_AUTH_MD5;
-  }
-
-  if ((cf->auth_type == BGP_AUTH_MD5) != !!cf->password)
-    cf_error("MD5 authentication and password option should be used together");
-
-  if ((cf->auth_type == BGP_AUTH_AO) != !!cf->ao_keys)
-    cf_error("AO authentication and keys option should be used together");
-
-  if ((cf->auth_type == BGP_AUTH_AO) && cf->remote_range)
-    cf_error("AO authentication not supported on dynamic BGP sessions");
 
   struct bgp_channel_config *cc;
   BGP_CF_WALK_CHANNELS(cf, cc)
@@ -2663,12 +2081,6 @@ bgp_postconfig(struct proto_config *CF)
     if (cc->llgr_time == ~0U)
       cc->llgr_time = cf->llgr_time;
 
-    if (cc->min_llgr_time == ~0U)
-      cc->min_llgr_time = cf->min_llgr_time;
-
-    if (cc->max_llgr_time == ~0U)
-      cc->max_llgr_time = cf->max_llgr_time;
-
     /* AIGP enabled by default on interior sessions */
     if (cc->aigp == 0xff)
       cc->aigp = interior;
@@ -2708,17 +2120,6 @@ bgp_postconfig(struct proto_config *CF)
 
     if (cc->secondary && !cc->c.table->sorted)
       cf_error("BGP with secondary option requires sorted table");
-
-    if (cc->require_ext_next_hop && !cc->ext_next_hop)
-      cf_warn("Extended next hop required but not enabled");
-
-    if (cc->require_add_path && !cc->add_path)
-      cf_warn("ADD-PATH required but not enabled");
-
-    if (cc->min_llgr_time > cc->max_llgr_time)
-      cf_error("Min long-lived stale time (%u) exceeds max long-lived stale time (%u)",
-	       cc->min_llgr_time, cc->max_llgr_time);
-
   }
 }
 
@@ -2732,9 +2133,6 @@ bgp_reconfigure(struct proto *P, struct proto_config *CF)
   if (proto_get_router_id(CF) != p->local_id)
     return 0;
 
-  if (bstrcmp(proto_get_hostname(CF), p->hostname))
-    return 0;
-
   int same = !memcmp(((byte *) old) + sizeof(struct proto_config),
 		     ((byte *) new) + sizeof(struct proto_config),
 		     // password item is last and must be checked separately
@@ -2745,9 +2143,6 @@ bgp_reconfigure(struct proto *P, struct proto_config *CF)
     && !bstrcmp(old->dynamic_name, new->dynamic_name)
     && (old->dynamic_name_digits == new->dynamic_name_digits);
 
-  /* Reconfigure TCP-AP */
-  same = same && bgp_reconfigure_ao_keys(p, new);
-
   /* FIXME: Move channel reconfiguration to generic protocol code ? */
   struct channel *C, *C2;
   struct bgp_channel_config *cc;
@@ -2755,48 +2150,30 @@ bgp_reconfigure(struct proto *P, struct proto_config *CF)
   WALK_LIST(C, p->p.channels)
     C->stale = 1;
 
-  /* Reconfigure BGP channels */
   BGP_CF_WALK_CHANNELS(new, cc)
   {
     C = (struct channel *) bgp_find_channel(p, cc->afi);
     same = proto_configure_channel(P, &C, &cc->c) && same;
   }
 
-  /* Reconfigure MPLS channel */
-  same = proto_configure_channel(P, &P->mpls_channel, proto_cf_mpls_channel(CF)) && same;
-
   WALK_LIST_DELSAFE(C, C2, p->p.channels)
     if (C->stale)
       same = proto_configure_channel(P, &C, NULL) && same;
 
+  if (same && (p->start_state > BSS_PREPARE))
+    bgp_update_bfd(p, new->bfd);
+
+  /* We should update our copy of configuration ptr as old configuration will be freed */
+  if (same)
+    p->cf = new;
+
   /* Reset name counter */
   p->dynamic_name_counter = 0;
 
-  if (!same)
-    return 0;
-
-  /* We should update our copy of configuration ptr as old configuration will be freed */
-  p->cf = new;
-  p->hostname = proto_get_hostname(CF);
-
-  /* Check whether existing connections are compatible with required capabilities */
-  struct bgp_conn *ci = &p->incoming_conn;
-  if (((ci->state == BS_OPENCONFIRM) || (ci->state == BS_ESTABLISHED)) && !bgp_check_capabilities(ci))
-    return 0;
-
-  struct bgp_conn *co = &p->outgoing_conn;
-  if (((co->state == BS_OPENCONFIRM) || (co->state == BS_ESTABLISHED)) && !bgp_check_capabilities(co))
-    return 0;
-
-  proto_setup_mpls_map(P, RTS_BGP, 1);
-
-  if (p->start_state > BSS_PREPARE)
-    bgp_update_bfd(p, new->bfd);
-
-  return 1;
+  return same;
 }
 
-#define TABLE(cf, NAME) ((cf)->NAME ? (cf)->NAME->table : NULL)
+#define TABLE(cf, NAME) ((cf)->NAME ? (cf)->NAME->table : NULL )
 
 static int
 bgp_channel_reconfigure(struct channel *C, struct channel_config *CC, int *import_changed, int *export_changed)
@@ -2820,10 +2197,6 @@ bgp_channel_reconfigure(struct channel *C, struct channel_config *CC, int *impor
       (TABLE(new, base_table) != TABLE(old, base_table)))
     return 0;
 
-  if (c->stale_time && ((new->min_llgr_time > c->stale_time) ||
-			(new->max_llgr_time < c->stale_time)))
-    return 0;
-
   if (new->mandatory && !old->mandatory && (C->channel_state != CS_UP))
     return 0;
 
@@ -2842,7 +2215,6 @@ bgp_channel_reconfigure(struct channel *C, struct channel_config *CC, int *impor
   if (!ipa_equal(new->next_hop_addr, old->next_hop_addr) ||
       (new->next_hop_self != old->next_hop_self) ||
       (new->next_hop_keep != old->next_hop_keep) ||
-      (new->llnh_format != old->llnh_format) ||
       (new->aigp != old->aigp) ||
       (new->aigp_originate != old->aigp_originate))
     *export_changed = 1;
@@ -2889,13 +2261,12 @@ bgp_error(struct bgp_conn *c, uint code, uint subcode, byte *data, int len)
 
   bgp_log_error(p, BE_BGP_TX, "Error", code, subcode, data, ABS(len));
   bgp_store_error(p, c, BE_BGP_TX, (code << 16) | subcode);
+  bgp_conn_enter_close_state(c);
 
   c->notify_code = code;
   c->notify_subcode = subcode;
   c->notify_data = data;
   c->notify_size = (len > 0) ? len : 0;
-
-  bgp_conn_enter_close_state(c);
   bgp_schedule_packet(c, NULL, PKT_NOTIFICATION);
 
   if (code != 6)
@@ -2933,10 +2304,10 @@ bgp_store_error(struct bgp_proto *p, struct bgp_conn *c, u8 class, u32 code)
 }
 
 static char *bgp_state_names[] = { "Idle", "Connect", "Active", "OpenSent", "OpenConfirm", "Established", "Close" };
-static char *bgp_err_classes[] = { "", "Error: ", "Socket: ", "Received: ", "BGP Error: ", "Automatic shutdown: ", "" };
-static char *bgp_misc_errors[] = { "", "Neighbor lost", "Invalid next hop", "Authentication failed", "No listening socket", "Link down", "BFD session down", "Graceful restart" };
-static char *bgp_auto_errors[] = { "", "Route limit exceeded" };
-static char *bgp_gr_states[] = { "None", "Regular", "Long-lived" };
+static char *bgp_err_classes[] = { "", "Error: ", "Socket: ", "Received: ", "BGP Error: ", "Automatic shutdown: ", ""};
+static char *bgp_misc_errors[] = { "", "Neighbor lost", "Invalid next hop", "Kernel MD5 auth failed", "No listening socket", "Link down", "BFD session down", "Graceful restart"};
+static char *bgp_auto_errors[] = { "", "Route limit exceeded"};
+static char *bgp_gr_states[] = { "None", "Regular", "Long-lived"};
 
 static const char *
 bgp_last_errmsg(struct bgp_proto *p)
@@ -3206,20 +2577,6 @@ bgp_show_proto_info(struct proto *P)
 	    tm_remains(p->conn->hold_timer), p->conn->hold_time);
     cli_msg(-1006, "    Keepalive timer:  %t/%u",
 	    tm_remains(p->conn->keepalive_timer), p->conn->keepalive_time);
-    cli_msg(-1006, "    Send hold timer:  %t/%u",
-	    tm_remains(p->conn->send_hold_timer), p->conn->send_hold_time);
-
-    if (!EMPTY_LIST(p->ao.keys))
-    {
-      struct ao_info info;
-      sk_get_ao_info(p->conn->sk, &info);
-
-      cli_msg(-1006, "    TCP-AO:");
-      cli_msg(-1006, "      Current key:    %i", info.current_key);
-      cli_msg(-1006, "      RNext key:      %i", info.rnext_key);
-      cli_msg(-1006, "      Good packets:   %lu", info.pkt_good);
-      cli_msg(-1006, "      Bad packets:    %lu", info.pkt_bad);
-    }
   }
 
 #if 0
@@ -3265,23 +2622,19 @@ bgp_show_proto_info(struct proto *P)
 	  cli_msg(-1006, "    BGP Next hop:   %I %I", c->next_hop_addr, c->link_addr);
       }
 
-      /* After channel is deconfigured, these pointers are no longer valid */
-      if (!p->p.reconfiguring || (c->c.channel_state != CS_DOWN))
-      {
-	if (c->igp_table_ip4)
-	  cli_msg(-1006, "    IGP IPv4 table: %s", c->igp_table_ip4->name);
+      if (c->igp_table_ip4)
+	cli_msg(-1006, "    IGP IPv4 table: %s", c->igp_table_ip4->name);
 
-	if (c->igp_table_ip6)
-	  cli_msg(-1006, "    IGP IPv6 table: %s", c->igp_table_ip6->name);
+      if (c->igp_table_ip6)
+	cli_msg(-1006, "    IGP IPv6 table: %s", c->igp_table_ip6->name);
 
-	if (c->base_table)
-	  cli_msg(-1006, "    Base table:     %s", c->base_table->name);
-      }
+      if (c->base_table)
+	cli_msg(-1006, "    Base table:     %s", c->base_table->name);
     }
   }
 }
 
-const struct channel_class channel_bgp = {
+struct channel_class channel_bgp = {
   .channel_size =	sizeof(struct bgp_channel),
   .config_size =	sizeof(struct bgp_channel_config),
   .init =		bgp_channel_init,
@@ -3296,7 +2649,7 @@ struct protocol proto_bgp = {
   .template = 		"bgp%d",
   .class =		PROTOCOL_BGP,
   .preference = 	DEF_PREF_BGP,
-  .channel_mask =	NB_IP | NB_VPN | NB_FLOW | NB_MPLS,
+  .channel_mask =	NB_IP | NB_VPN | NB_FLOW,
   .proto_size =		sizeof(struct bgp_proto),
   .config_size =	sizeof(struct bgp_config),
   .postconfig =		bgp_postconfig,
