@@ -167,17 +167,15 @@ bgp_create_notification(struct bgp_conn *conn, byte *buf)
   buf[0] = conn->notify_code;
   buf[1] = conn->notify_subcode;
   memcpy(buf+2, conn->notify_data, conn->notify_size);
+  bmp_peer_down(p, BE_NONE, buf, conn->notify_size + 2);
   return buf + 2 + conn->notify_size;
 }
 
 
 /* Capability negotiation as per RFC 5492 */
 
-static const struct bgp_af_caps dummy_af_caps = { };
-static const struct bgp_af_caps basic_af_caps = { .ready = 1 };
-
-static const struct bgp_af_caps *
-bgp_find_af_caps_(struct bgp_caps *caps, u32 afi)
+const struct bgp_af_caps *
+bgp_find_af_caps(struct bgp_caps *caps, u32 afi)
 {
   struct bgp_af_caps *ac;
 
@@ -188,23 +186,6 @@ bgp_find_af_caps_(struct bgp_caps *caps, u32 afi)
   return NULL;
 }
 
-const struct bgp_af_caps *
-bgp_find_af_caps(struct bgp_caps *caps, u32 afi)
-{
-  const struct bgp_af_caps *ac = bgp_find_af_caps_(caps, afi);
-
-  /* Return proper capability if found */
-  if (ac)
-    return ac;
-
-  /* Use default if capabilities were not announced */
-  if (!caps->length && (afi == BGP_AF_IPV4))
-    return &basic_af_caps;
-
-  /* Ignore AFIs that were not announced in multiprotocol capability */
-  return &dummy_af_caps;
-}
-
 static struct bgp_af_caps *
 bgp_get_af_caps(struct bgp_caps **pcaps, u32 afi)
 {
@@ -234,13 +215,6 @@ bgp_af_caps_cmp(const void *X, const void *Y)
   return (x->afi < y->afi) ? -1 : (x->afi > y->afi) ? 1 : 0;
 }
 
-struct bgp_caps *
-bgp_alloc_capabilities(struct bgp_proto *p, int n)
-{
-  struct bgp_caps *caps = mb_allocz(p->p.pool, sizeof(struct bgp_caps) + n * sizeof(struct bgp_af_caps));
-  caps->role = BGP_ROLE_UNDEFINED;
-  return caps;
-}
 
 void
 bgp_prepare_capabilities(struct bgp_conn *conn)
@@ -253,19 +227,19 @@ bgp_prepare_capabilities(struct bgp_conn *conn)
   if (!p->cf->capabilities)
   {
     /* Just prepare empty local_caps */
-    conn->local_caps = bgp_alloc_capabilities(p, 0);
+    conn->local_caps = mb_allocz(p->p.pool, sizeof(struct bgp_caps));
     return;
   }
 
   /* Prepare bgp_caps structure */
   int n = list_length(&p->p.channels);
-  caps = bgp_alloc_capabilities(p, n);
+  caps = mb_allocz(p->p.pool, sizeof(struct bgp_caps) + n * sizeof(struct bgp_af_caps));
   conn->local_caps = caps;
 
   caps->as4_support = p->cf->enable_as4;
   caps->ext_messages = p->cf->enable_extended_messages;
   caps->route_refresh = p->cf->enable_refresh;
-  caps->enhanced_refresh = p->cf->enable_refresh && p->cf->enable_enhanced_refresh;
+  caps->enhanced_refresh = p->cf->enable_refresh;
   caps->role = p->cf->local_role;
 
   if (caps->as4_support)
@@ -281,11 +255,11 @@ bgp_prepare_capabilities(struct bgp_conn *conn)
   if (p->cf->llgr_mode)
     caps->llgr_aware = 1;
 
-  if (p->cf->enable_hostname && p->hostname)
+  if (p->cf->enable_hostname && config->hostname)
   {
-    size_t length = strlen(p->hostname);
+    size_t length = strlen(config->hostname);
     char *hostname = mb_allocz(p->p.pool, length+1);
-    memcpy(hostname, p->hostname, length+1);
+    memcpy(hostname, config->hostname, length+1);
     caps->hostname = hostname;
   }
 
@@ -490,7 +464,10 @@ bgp_read_capabilities(struct bgp_conn *conn, byte *pos, int len)
   u32 af;
 
   if (!conn->remote_caps)
-    caps = bgp_alloc_capabilities(p, 1);
+  {
+    caps = mb_allocz(p->p.pool, sizeof(struct bgp_caps) + sizeof(struct bgp_af_caps));
+    caps->role = BGP_ROLE_UNDEFINED;
+  }
   else
   {
     caps = conn->remote_caps;
@@ -526,7 +503,7 @@ bgp_read_capabilities(struct bgp_conn *conn, byte *pos, int len)
       caps->route_refresh = 1;
       break;
 
-    case  5: /* Extended next hop encoding capability, RFC 8950 */
+    case  5: /* Extended next hop encoding capability, RFC 5549 */
       if (cl % 6)
 	goto err;
 
@@ -627,7 +604,7 @@ bgp_read_capabilities(struct bgp_conn *conn, byte *pos, int len)
       caps->enhanced_refresh = 1;
       break;
 
-    case 71: /* Long lived graceful restart capability, RFC 9494 */
+    case 71: /* Long lived graceful restart capability, RFC draft */
       if (cl % 7)
 	goto err;
 
@@ -694,7 +671,7 @@ err:
   return -1;
 }
 
-int
+static int
 bgp_check_capabilities(struct bgp_conn *conn)
 {
   struct bgp_proto *p = conn->bgp;
@@ -706,54 +683,21 @@ bgp_check_capabilities(struct bgp_conn *conn)
   /* This is partially overlapping with bgp_conn_enter_established_state(),
      but we need to run this just after we receive OPEN message */
 
-  if (p->cf->require_refresh && !remote->route_refresh)
-    return 0;
-
-  if (p->cf->require_enhanced_refresh && !remote->enhanced_refresh)
-    return 0;
-
-  if (p->cf->require_as4 && !remote->as4_support)
-    return 0;
-
-  if (p->cf->require_extended_messages && !remote->ext_messages)
-    return 0;
-
-  if (p->cf->require_hostname && !remote->hostname)
-    return 0;
-
-  if (p->cf->require_gr && !remote->gr_aware)
-    return 0;
-
-  if (p->cf->require_llgr && !remote->llgr_aware)
-    return 0;
-
-  /* No check for require_roles, as it uses error code 2.11 instead of 2.7 */
-
   BGP_WALK_CHANNELS(p, c)
   {
     const struct bgp_af_caps *loc = bgp_find_af_caps(local,  c->afi);
     const struct bgp_af_caps *rem = bgp_find_af_caps(remote, c->afi);
 
     /* Find out whether this channel will be active */
-    int active = loc->ready && rem->ready;
+    int active = loc && loc->ready &&
+      ((rem && rem->ready) || (!remote->length && (c->afi == BGP_AF_IPV4)));
 
     /* Mandatory must be active */
     if (c->cf->mandatory && !active)
       return 0;
 
     if (active)
-    {
-      if (c->cf->require_ext_next_hop && !rem->ext_next_hop)
-	return 0;
-
-      if (c->cf->require_add_path && (loc->add_path & BGP_ADD_PATH_RX) && !(rem->add_path & BGP_ADD_PATH_TX))
-	return 0;
-
-      if (c->cf->require_add_path && (loc->add_path & BGP_ADD_PATH_TX) && !(rem->add_path & BGP_ADD_PATH_RX))
-	return 0;
-
       count++;
-    }
   }
 
   /* We need at least one channel active */
@@ -819,7 +763,7 @@ bgp_read_options(struct bgp_conn *conn, byte *pos, uint len, uint rest)
 
   /* Prepare empty caps if no capability option was announced */
   if (!conn->remote_caps)
-    conn->remote_caps = bgp_alloc_capabilities(p, 0);
+    conn->remote_caps = mb_allocz(p->p.pool, sizeof(struct bgp_caps));
 
   return 0;
 
@@ -828,14 +772,6 @@ err:
   return -1;
 }
 
-static byte *
-bgp_copy_open(struct bgp_proto *p, const byte *pkt, uint len)
-{
-  char *buf = mb_alloc(p->p.pool, len - BGP_HEADER_LENGTH);
-  memcpy(buf, pkt + BGP_HEADER_LENGTH, len - BGP_HEADER_LENGTH);
-  return buf;
-}
-
 static byte *
 bgp_create_open(struct bgp_conn *conn, byte *buf)
 {
@@ -910,9 +846,6 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
   id = get_u32(pkt+24);
   BGP_TRACE(D_PACKETS, "Got OPEN(as=%d,hold=%d,id=%R)", asn, hold, id);
 
-  conn->remote_open_msg = bgp_copy_open(p, pkt, len);
-  conn->remote_open_length = len - BGP_HEADER_LENGTH;
-
   if (bgp_read_options(conn, pkt+29, pkt[28], len-29) < 0)
     return;
 
@@ -926,10 +859,6 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
     (p->cf->keepalive_time * hold_time / p->cf->hold_time) :
     hold_time / 3;
 
-  uint send_hold_time = (p->cf->send_hold_time >= 0) ?
-    (p->cf->send_hold_time * hold_time / p->cf->hold_time) :
-    2 * hold_time;
-
   /* Keepalive time might be rounded down to zero */
   if (hold_time && !keepalive_time)
     keepalive_time = 1;
@@ -943,7 +872,7 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
   if (!id || (p->is_internal && id == p->local_id))
   { bgp_error(conn, 2, 3, pkt+24, -4); return; }
 
-  /* RFC 5492 5 - check for required capabilities */
+  /* RFC 5492 4 - check for required capabilities */
   if (p->cf->capabilities && !bgp_check_capabilities(conn))
   { bgp_error(conn, 2, 7, NULL, 0); return; }
 
@@ -1038,7 +967,6 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
   /* Update our local variables */
   conn->hold_time = hold_time;
   conn->keepalive_time = keepalive_time;
-  conn->send_hold_time = send_hold_time;
   conn->as4_session = conn->local_caps->as4_support && caps->as4_support;
   conn->ext_messages = conn->local_caps->ext_messages && caps->ext_messages;
   p->remote_id = id;
@@ -1048,8 +976,8 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
 
   bgp_schedule_packet(conn, NULL, PKT_KEEPALIVE);
   bgp_start_timer(conn->hold_timer, conn->hold_time);
-  bgp_start_timer(conn->send_hold_timer, conn->send_hold_time);
   bgp_conn_enter_openconfirm_state(conn);
+  bmp_put_recv_bgp_open_msg(p, pkt, len);
 }
 
 
@@ -1238,7 +1166,7 @@ bgp_use_gateway(struct bgp_export_state *s)
     return 0;
 
   /* Do not use gateway from different VRF */
-  if (p->p.vrf_set && ra->nh.iface && !if_in_vrf(ra->nh.iface, p->p.vrf))
+  if (p->p.vrf_set && ra->nh.iface && (p->p.vrf != ra->nh.iface->master))
     return 0;
 
   /* Use it when exported to internal peers */
@@ -1276,10 +1204,11 @@ bgp_update_next_hop_ip(struct bgp_export_state *s, eattr *a, ea_list **to)
       bgp_set_attr_data(to, s->pool, BA_NEXT_HOP, 0, nh, ipa_nonzero(nh[1]) ? 32 : 16);
       s->local_next_hop = 1;
 
+      /* TODO: Use local MPLS assigned label */
       if (s->mpls)
       {
-	u32 label = ea_get_int(s->route->attrs->eattrs, EA_MPLS_LABEL, BGP_MPLS_NULL);
-	bgp_set_attr_data(to, s->pool, BA_MPLS_LABEL_STACK, 0, &label, 4);
+	u32 implicit_null = BGP_MPLS_NULL;
+	bgp_set_attr_data(to, s->pool, BA_MPLS_LABEL_STACK, 0, &implicit_null, 4);
       }
       else
 	bgp_unset_attr(to, s->pool, BA_MPLS_LABEL_STACK);
@@ -1313,37 +1242,6 @@ bgp_update_next_hop_ip(struct bgp_export_state *s, eattr *a, ea_list **to)
     REJECT(NO_LABEL_STACK);
 }
 
-static uint
-bgp_prepare_link_local_next_hop(struct bgp_write_state *s, ip_addr *nh)
-{
-  /*
-   * We have link-local next-hop in nh[1]
-   *
-   * Possible variants:
-   * [ ::, fe80::XX ] - BIRD's default/internal (LLNH_NATIVE)
-   * [ fe80::XX ] - draft-white-linklocal-capability (LLNH_SINGLE)
-   * [ fe80::XX, fe80::XX ] - Used in JunoOS (LLNH_DOUBLE)
-   */
-
-  switch (s->llnh_format)
-  {
-  case LLNH_NATIVE:
-    return 32;
-
-  case LLNH_SINGLE:
-    nh[0] = nh[1];
-    return 16;
-
-  case LLNH_DOUBLE:
-    nh[0] = nh[1];
-    return 32;
-
-  default:
-    ASSERT(0);
-    return 32;
-  }
-}
-
 static uint
 bgp_encode_next_hop_ip(struct bgp_write_state *s, eattr *a, byte *buf, uint size UNUSED)
 {
@@ -1355,7 +1253,7 @@ bgp_encode_next_hop_ip(struct bgp_write_state *s, eattr *a, byte *buf, uint size
 
   /*
    * Both IPv4 and IPv6 next hops can be used (with ext_next_hop enabled). This
-   * is specified in RFC 8950 for IPv4 and in RFC 4798 for IPv6. The difference
+   * is specified in RFC 5549 for IPv4 and in RFC 4798 for IPv6. The difference
    * is that IPv4 address is directly encoded with IPv4 NLRI, but as IPv4-mapped
    * IPv6 address with IPv6 NLRI.
    */
@@ -1366,13 +1264,6 @@ bgp_encode_next_hop_ip(struct bgp_write_state *s, eattr *a, byte *buf, uint size
     return 4;
   }
 
-  /* Handle variants of link-local-only next hop */
-  if (ipa_zero(nh[0]) && (len == 32))
-  {
-    nh = alloca_copy(nh, 32);
-    len = bgp_prepare_link_local_next_hop(s, nh);
-  }
-
   put_ip6(buf, ipa_to_ip6(nh[0]));
 
   if (len == 32)
@@ -1437,7 +1328,7 @@ bgp_encode_next_hop_vpn(struct bgp_write_state *s, eattr *a, byte *buf, uint siz
 
   /*
    * Both IPv4 and IPv6 next hops can be used (with ext_next_hop enabled). This
-   * is specified in RFC 8950 for VPNv4 and in RFC 4659 for VPNv6. The difference
+   * is specified in RFC 5549 for VPNv4 and in RFC 4659 for VPNv6. The difference
    * is that IPv4 address is directly encoded with VPNv4 NLRI, but as IPv4-mapped
    * IPv6 address with VPNv6 NLRI.
    */
@@ -1449,13 +1340,6 @@ bgp_encode_next_hop_vpn(struct bgp_write_state *s, eattr *a, byte *buf, uint siz
     return 12;
   }
 
-  /* Handle variants of link-local-only next hop */
-  if (ipa_zero(nh[0]) && (len == 32))
-  {
-    nh = alloca_copy(nh, 32);
-    len = bgp_prepare_link_local_next_hop(s, nh);
-  }
-
   put_u64(buf, 0); /* VPN RD is 0 */
   put_ip6(buf+8, ipa_to_ip6(nh[0]));
 
@@ -1493,9 +1377,6 @@ bgp_decode_next_hop_vpn(struct bgp_parse_state *s, byte *data, uint len, rta *a)
     nh[0] = ipa_from_ip6(get_ip6(data+8));
     nh[1] = ipa_from_ip6(get_ip6(data+32));
 
-    if (ipa_is_link_local(nh[0]))
-    { nh[1] = nh[0]; nh[0] = IPA_NONE; }
-
     if (ipa_is_ip4(nh[0]) || !ip6_is_link_local(nh[1]))
       nh[1] = IPA_NONE;
   }
@@ -1532,7 +1413,7 @@ bgp_decode_next_hop_none(struct bgp_parse_state *s UNUSED, byte *data UNUSED, ui
   /*
    * Although we expect no next hop and RFC 7606 7.11 states that attribute
    * MP_REACH_NLRI with unexpected next hop length is considered malformed,
-   * FlowSpec RFC 8955 4 states that next hop shall be ignored on receipt.
+   * FlowSpec RFC 5575 4 states that next hop shall be ignored on receipt.
    */
 
   return;
@@ -1855,7 +1736,7 @@ bgp_encode_nlri_vpn4(struct bgp_write_state *s, struct bgp_bucket *buck, byte *b
       bgp_encode_mpls_labels(s, s->mpls_labels, &pos, &size, pos - 1);
 
     /* Encode route distinguisher */
-    put_rd(pos, net->rd);
+    put_u64(pos, net->rd);
     ADVANCE(pos, size, 8);
 
     /* Encode prefix body */
@@ -1906,7 +1787,7 @@ bgp_decode_nlri_vpn4(struct bgp_parse_state *s, byte *pos, uint len, rta *a)
     if (l < 64)
       bgp_parse_error(s, 1);
 
-    vpn_rd rd = get_rd(pos);
+    u64 rd = get_u64(pos);
     ADVANCE(pos, len, 8);
     l -= 64;
 
@@ -1955,7 +1836,7 @@ bgp_encode_nlri_vpn6(struct bgp_write_state *s, struct bgp_bucket *buck, byte *b
       bgp_encode_mpls_labels(s, s->mpls_labels, &pos, &size, pos - 1);
 
     /* Encode route distinguisher */
-    put_rd(pos, net->rd);
+    put_u64(pos, net->rd);
     ADVANCE(pos, size, 8);
 
     /* Encode prefix body */
@@ -2006,7 +1887,7 @@ bgp_decode_nlri_vpn6(struct bgp_parse_state *s, byte *pos, uint len, rta *a)
     if (l < 64)
       bgp_parse_error(s, 1);
 
-    vpn_rd rd = get_rd(pos);
+    u64 rd = get_u64(pos);
     ADVANCE(pos, len, 8);
     l -= 64;
 
@@ -2530,17 +2411,14 @@ bgp_create_mp_unreach(struct bgp_write_state *s, struct bgp_bucket *buck, byte *
 #ifdef CONFIG_BMP
 
 static byte *
-bgp_create_update_bmp(struct bgp_channel *c, byte *buf, byte *end, struct bgp_bucket *buck, bool update)
+bgp_create_update_bmp(struct bgp_channel *c, byte *buf, struct bgp_bucket *buck, bool update)
 {
   struct bgp_proto *p = (void *) c->c.proto;
-  byte *res = NULL;
-
-  struct lp_state tmpp;
-  lp_save(tmp_linpool, &tmpp);
+  byte *end = buf + (BGP_MAX_EXT_MSG_LENGTH - BGP_HEADER_LENGTH);
+  /* FIXME: must be a bit shorter */
 
   struct bgp_caps *peer = p->conn->remote_caps;
   const struct bgp_af_caps *rem = bgp_find_af_caps(peer, c->afi);
-
   struct bgp_write_state s = {
     .proto = p,
     .channel = c,
@@ -2554,27 +2432,38 @@ bgp_create_update_bmp(struct bgp_channel *c, byte *buf, byte *end, struct bgp_bu
 
   if (!update)
   {
-    res = !s.mp_reach ?
+    return !s.mp_reach ?
       bgp_create_ip_unreach(&s, buck, buf, end):
       bgp_create_mp_unreach(&s, buck, buf, end);
   }
   else
   {
-    res = !s.mp_reach ?
+    return !s.mp_reach ?
       bgp_create_ip_reach(&s, buck, buf, end):
       bgp_create_mp_reach(&s, buck, buf, end);
   }
-
-  lp_restore(tmp_linpool, &tmpp);
-
-  return res;
 }
 
-byte *
-bgp_bmp_encode_rte(struct bgp_channel *c, byte *buf, byte *end, const net_addr *n,
-		   const struct rte *new, const struct rte_src *src)
+static byte *
+bgp_bmp_prepare_bgp_hdr(byte *buf, const u16 msg_size, const u8 msg_type)
 {
-//  struct bgp_proto *p = (void *) c->c.proto;
+  memset(buf + BGP_MSG_HDR_MARKER_POS, 0xff, BGP_MSG_HDR_MARKER_SIZE);
+  put_u16(buf + BGP_MSG_HDR_LENGTH_POS, msg_size);
+  put_u8(buf + BGP_MSG_HDR_TYPE_POS, msg_type);
+
+  return buf + BGP_MSG_HDR_TYPE_POS + BGP_MSG_HDR_TYPE_SIZE;
+}
+
+void
+bgp_rte_update_in_notify(struct channel *C, const net_addr *n,
+			 const struct rte *new, const struct rte_src *src)
+{
+//  struct bgp_proto *p = (void *) C->proto;
+  struct bgp_channel *c = (void *) C;
+
+  byte buf[BGP_MAX_EXT_MSG_LENGTH];
+  byte *pkt = buf + BGP_HEADER_LENGTH;
+
   ea_list *attrs = new ? new->attrs->eattrs : NULL;
   uint ea_size = new ? (sizeof(ea_list) + attrs->count * sizeof(eattr)) : 0;
   uint bucket_size = sizeof(struct bgp_bucket) + ea_size;
@@ -2591,11 +2480,16 @@ bgp_bmp_encode_rte(struct bgp_channel *c, byte *buf, byte *end, const net_addr *
   /* Sham prefix */
   struct bgp_prefix *px = alloca(prefix_size);
   *px = (struct bgp_prefix) { };
-  px->path_id = (u32) src->private_id;
+  px->path_id = src->private_id;
   net_copy(px->net, n);
   add_tail(&b->prefixes, &px->buck_node);
 
-  return bgp_create_update_bmp(c, buf, end, b, !!new);
+  byte *end = bgp_create_update_bmp(c, pkt, b, !!new);
+  if (!end)
+    return;
+
+  bgp_bmp_prepare_bgp_hdr(buf, end - buf, PKT_UPDATE);
+  bmp_route_monitor_put_update_in_pre_msg(buf, end - buf);
 }
 
 #endif /* CONFIG_BMP */
@@ -2622,7 +2516,6 @@ again: ;
     .mp_reach = (c->afi != BGP_AF_IPV4) || c->ext_next_hop,
     .as4_session = p->as4_session,
     .add_path = c->add_path_tx,
-    .llnh_format = c->cf->llnh_format,
     .mpls = c->desc->mpls,
   };
 
@@ -2703,14 +2596,6 @@ bgp_create_mp_end_mark(struct bgp_channel *c, byte *buf)
 }
 
 byte *
-bgp_create_end_mark_(struct bgp_channel *c, byte *buf)
-{
-  return (c->afi == BGP_AF_IPV4) ?
-    bgp_create_ip_end_mark(c, buf):
-    bgp_create_mp_end_mark(c, buf);
-}
-
-static byte *
 bgp_create_end_mark(struct bgp_channel *c, byte *buf)
 {
   struct bgp_proto *p = (void *) c->c.proto;
@@ -2718,7 +2603,9 @@ bgp_create_end_mark(struct bgp_channel *c, byte *buf)
   BGP_TRACE(D_PACKETS, "Sending END-OF-RIB");
   p->stats.tx_updates++;
 
-  return bgp_create_end_mark_(c, buf);
+  return (c->afi == BGP_AF_IPV4) ?
+    bgp_create_ip_end_mark(c, buf):
+    bgp_create_mp_end_mark(c, buf);
 }
 
 static inline void
@@ -2859,6 +2746,8 @@ bgp_rx_update(struct bgp_conn *conn, byte *pkt, uint len)
   s.ip_reach_len = len - pos;
   s.ip_reach_nlri = pkt + pos;
 
+  bmp_route_monitor_update_in_pre_begin();
+
   if (s.attr_len)
     ea = bgp_decode_attrs(&s, s.attrs, s.attr_len);
   else
@@ -2889,6 +2778,9 @@ bgp_rx_update(struct bgp_conn *conn, byte *pkt, uint len)
     bgp_decode_nlri(&s, s.mp_reach_af, s.mp_reach_nlri, s.mp_reach_len,
 		    ea, s.mp_next_hop_data, s.mp_next_hop_len);
 
+  bmp_route_monitor_update_in_pre_commit(p);
+  bmp_route_monitor_update_in_pre_end();
+
 done:
   rta_free(s.cached_rta);
   lp_restore(tmp_linpool, &tmpp);
@@ -3092,15 +2984,11 @@ bgp_send(struct bgp_conn *conn, uint type, uint len)
   conn->bgp->stats.tx_messages++;
   conn->bgp->stats.tx_bytes += len;
 
-  memset(buf, 0xff, BGP_HDR_MARKER_LENGTH);
+  memset(buf, 0xff, 16);		/* Marker */
   put_u16(buf+16, len);
   buf[18] = type;
 
-  int success = sk_send(sk, len);
-  if (success && ((conn->state == BS_ESTABLISHED) || (conn->state == BS_OPENCONFIRM)))
-    bgp_start_timer(conn->send_hold_timer, conn->send_hold_time);
-
-  return success;
+  return sk_send(sk, len);
 }
 
 /**
@@ -3144,11 +3032,12 @@ bgp_fire_tx(struct bgp_conn *conn)
   {
     conn->packets_to_send &= ~(1 << PKT_OPEN);
     end = bgp_create_open(conn, pkt);
-
-    conn->local_open_msg = bgp_copy_open(p, buf, end - buf);
-    conn->local_open_length = end - buf - BGP_HEADER_LENGTH;
-
-    return bgp_send(conn, PKT_OPEN, end - buf);
+    int rv = bgp_send(conn, PKT_OPEN, end - buf);
+    if (rv >= 0)
+    {
+      bmp_put_sent_bgp_open_msg(p, pkt, end - buf);
+    }
+    return rv;
   }
   else if (s & (1 << PKT_KEEPALIVE))
   {
@@ -3260,10 +3149,6 @@ bgp_tx(sock *sk)
 {
   struct bgp_conn *conn = sk->data;
 
-  /* Pending message was passed to kernel */
-  if ((conn->state == BS_ESTABLISHED) || (conn->state == BS_OPENCONFIRM))
-    bgp_start_timer(conn->send_hold_timer, conn->send_hold_time);
-
   DBG("BGP: TX hook\n");
   uint max = 1024;
   while (--max && (bgp_fire_tx(conn) > 0))
@@ -3319,8 +3204,7 @@ static struct {
   { 6, 7, "Connection collision resolution" },
   { 6, 8, "Out of Resources" },
   { 7, 0, "Invalid ROUTE-REFRESH message" }, /* [RFC7313] */
-  { 7, 1, "Invalid ROUTE-REFRESH message length" }, /* [RFC7313] */
-  { 8, 0, "Send hold timer expired" }, /* [RFC9687] */
+  { 7, 1, "Invalid ROUTE-REFRESH message length" } /* [RFC7313] */
 };
 
 /**
@@ -3345,7 +3229,7 @@ bgp_error_dsc(uint code, uint subcode)
   return buff;
 }
 
-/* RFC 9003 - shutdown communication message */
+/* RFC 8203 - shutdown communication message */
 static int
 bgp_handle_message(struct bgp_proto *p, byte *data, uint len, byte **bp)
 {
@@ -3400,15 +3284,15 @@ bgp_log_error(struct bgp_proto *p, u8 class, char *msg, uint code, uint subcode,
 	  goto done;
         }
 
-      /* RFC 9003 - shutdown communication */
+      /* RFC 8203 - shutdown communication */
       if (((code == 6) && ((subcode == 2) || (subcode == 4))))
 	if (bgp_handle_message(p, data, len, &t))
 	  goto done;
 
       *t++ = ':';
       *t++ = ' ';
-      if (len > 128)
-	len = 128;
+      if (len > 16)
+	len = 16;
       for (i=0; i<len; i++)
 	t += bsprintf(t, "%02x", data[i]);
     }
@@ -3434,11 +3318,6 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, uint len)
   bgp_log_error(p, BE_BGP_RX, "Received", code, subcode, pkt+21, len-21);
   bgp_store_error(p, conn, BE_BGP_RX, (code << 16) | subcode);
 
-  conn->notify_code = code;
-  conn->notify_subcode = subcode;
-  conn->notify_data = pkt+21;
-  conn->notify_size = len-21;
-
   bgp_conn_enter_close_state(conn);
   bgp_schedule_packet(conn, NULL, PKT_SCHEDULE_CLOSE);
 
@@ -3457,6 +3336,8 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, uint len)
       p->p.disabled = 1;
     }
   }
+
+  bmp_peer_down(p, BE_NONE, pkt, len);
 }
 
 static void
@@ -3521,7 +3402,6 @@ int
 bgp_rx(sock *sk, uint size)
 {
   struct bgp_conn *conn = sk->data;
-
   byte *pkt_start = sk->rbuf;
   byte *end = pkt_start + size;
   uint i, len;
