@@ -29,12 +29,10 @@
 #include "lib/socket.h"
 #include "lib/event.h"
 #include "lib/timer.h"
-#include "lib/tlists.h"
 #include "lib/string.h"
 #include "nest/route.h"
 #include "nest/protocol.h"
 #include "nest/iface.h"
-#include "nest/mpls.h"
 #include "nest/cli.h"
 #include "nest/locks.h"
 #include "conf/conf.h"
@@ -48,38 +46,21 @@
  *	Debugging
  */
 
-static void
-async_dump_report(struct dump_request *dr UNUSED, int state, const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  vlog(((state > 1000) ? L_ERR : L_INFO)[0], fmt, args);
-  va_end(args);
-}
-
-static void
-async_dump_run(struct dump_request *dreq)
-{
-  RDUMP("ASYNC STATE DUMP\n");
-
-  rdump(dreq, &root_pool);
-  sk_dump_all(dreq);
-  // XXXX tm_dump_all();
-  if_dump_all(dreq);
-  neigh_dump_all(dreq);
-  rta_dump_all(dreq);
-  rt_dump_all(dreq);
-  protos_dump_all(dreq);
-
-  debug("\n");
-}
-
 void
 async_dump(void)
 {
-  struct dump_request *dr = dump_to_file_init(0);
-  dr->report = async_dump_report;
-  dump_to_file_run(dr, "bird.dump", "async dump", async_dump_run);
+  debug("INTERNAL STATE DUMP\n\n");
+
+  rdump(&root_pool);
+  sk_dump_all();
+  // XXXX tm_dump_all();
+  if_dump_all();
+  neigh_dump_all();
+  rta_dump_all();
+  rt_dump_all();
+  protos_dump_all();
+
+  debug("\n");
 }
 
 /*
@@ -130,21 +111,21 @@ get_hostname(linpool *lp)
 #ifdef PATH_IPROUTE_DIR
 
 static inline void
-add_num_const(struct config *conf, char *name, int val, const char *file, const uint line)
+add_num_const(char *name, int val, const char *file, const uint line)
 {
   struct f_val *v = cfg_alloc(sizeof(struct f_val));
   *v = (struct f_val) { .type = T_INT, .val.i = val };
-  struct symbol *sym = cf_get_symbol(conf, name);
-  if (sym->class && cf_symbol_is_local(conf, sym))
+  struct symbol *sym = cf_get_symbol(name);
+  if (sym->class && cf_symbol_is_local(sym))
     cf_error("Error reading value for %s from %s:%d: already defined", name, file, line);
 
-  cf_define_symbol(conf, sym, SYM_CONSTANT | T_INT, val, v);
+  cf_define_symbol(sym, SYM_CONSTANT | T_INT, val, v);
 }
 
 /* the code of read_iproute_table() is based on
    rtnl_tab_initialize() from iproute2 package */
 static void
-read_iproute_table(struct config *conf, char *file, char *prefix, uint max)
+read_iproute_table(char *file, char *prefix, uint max)
 {
   char buf[512], namebuf[512];
   char *name;
@@ -181,7 +162,7 @@ read_iproute_table(struct config *conf, char *file, char *prefix, uint max)
       if ((*p < 'a' || *p > 'z') && (*p < 'A' || *p > 'Z') && (*p < '0' || *p > '9') && (*p != '_'))
 	*p = '_';
 
-    add_num_const(conf, namebuf, val, file, line);
+    add_num_const(namebuf, val, file, line);
   }
 
   fclose(fp);
@@ -201,8 +182,6 @@ cf_read(byte *dest, uint len, int fd)
   return l;
 }
 
-static void cli_preconfig(struct config *c);
-
 void
 sysdep_preconfig(struct config *c)
 {
@@ -212,22 +191,17 @@ sysdep_preconfig(struct config *c)
   c->watchdog_warning = UNIX_DEFAULT_WATCHDOG_WARNING;
 
 #ifdef PATH_IPROUTE_DIR
-  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_protos", "ipp_", 255);
-  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_realms", "ipr_", 0xffffffff);
-  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_scopes", "ips_", 255);
-  read_iproute_table(c, PATH_IPROUTE_DIR "/rt_tables", "ipt_", 0xffffffff);
+  read_iproute_table(PATH_IPROUTE_DIR "/rt_protos", "ipp_", 255);
+  read_iproute_table(PATH_IPROUTE_DIR "/rt_realms", "ipr_", 0xffffffff);
+  read_iproute_table(PATH_IPROUTE_DIR "/rt_scopes", "ips_", 255);
+  read_iproute_table(PATH_IPROUTE_DIR "/rt_tables", "ipt_", 0xffffffff);
 #endif
-
-  cli_preconfig(c);
 }
 
-static void cli_commit(struct config *new, struct config *old);
-
 int
-sysdep_commit(struct config *new, struct config *old)
+sysdep_commit(struct config *new, struct config *old UNUSED)
 {
   log_switch(0, &new->logfiles, new->syslog_name);
-  cli_commit(new, old);
   return 0;
 }
 
@@ -420,28 +394,9 @@ cmd_reconfig_status(void)
  *	Command-Line Interface
  */
 
-static struct cli_config initial_control_socket_config = {
-  .name = PATH_CONTROL_SOCKET,
-  .mode = 0660,
-};
-#define path_control_socket initial_control_socket_config.name
+static sock *cli_sk;
+static char *path_control_socket = PATH_CONTROL_SOCKET;
 
-static struct cli_config *main_control_socket_config = NULL;
-
-#define TLIST_PREFIX cli_listener
-#define TLIST_TYPE struct cli_listener
-#define TLIST_ITEM n
-#define TLIST_WANT_ADD_TAIL
-#define TLIST_WANT_WALK
-static struct cli_listener {
-  TLIST_DEFAULT_NODE;
-  sock *s;
-  struct cli_config *config;
-} *main_control_socket = NULL;
-
-#include "lib/tlists.h"
-
-static TLIST_LIST(cli_listener) cli_listeners;
 
 static void
 cli_write(cli *c)
@@ -555,7 +510,7 @@ cli_connect(sock *s, uint size UNUSED)
   s->rx_hook = cli_rx;
   s->tx_hook = cli_tx;
   s->err_hook = cli_err;
-  s->data = c = cli_new(s, ((struct cli_listener *) s->data)->config);
+  s->data = c = cli_new(s);
   s->pool = c->pool;		/* We need to have all the socket buffers allocated in the cli pool */
   s->fast_rx = 1;
   c->rx_pos = c->rx_buf;
@@ -563,116 +518,33 @@ cli_connect(sock *s, uint size UNUSED)
   return 1;
 }
 
-static struct cli_listener *
-cli_listen(struct cli_config *cf)
+static void
+cli_init_unix(uid_t use_uid, gid_t use_gid)
 {
-  struct cli_listener *l = mb_allocz(cli_pool, sizeof *l);
-  l->config = cf;
-  sock *s = l->s = sk_new(cli_pool);
+  sock *s;
+
+  cli_init();
+  s = cli_sk = sk_new(cli_pool);
   s->type = SK_UNIX_PASSIVE;
   s->rx_hook = cli_connect;
   s->err_hook = cli_connect_err;
-  s->data = l;
   s->rbsize = 1024;
   s->fast_rx = 1;
 
   /* Return value intentionally ignored */
-  unlink(cf->name);
+  unlink(path_control_socket);
 
-  if (sk_open_unix(s, cf->name) < 0)
-  {
-    log(L_ERR "Cannot create control socket %s: %m", cf->name);
-    return NULL;
-  }
+  if (sk_open_unix(s, path_control_socket) < 0)
+    die("Cannot create control socket %s: %m", path_control_socket);
 
-  if (cf->uid || cf->gid)
-    if (chown(cf->name, cf->uid, cf->gid) < 0)
-    {
-      log(L_ERR "Cannot chown control socket %s: %m", cf->name);
-      return NULL;
-    }
+  if (use_uid || use_gid)
+    if (chown(path_control_socket, use_uid, use_gid) < 0)
+      die("chown: %m");
 
-  if (chmod(cf->name, cf->mode) < 0)
-  {
-    log(L_ERR "Cannot chmod control socket %s: %m", cf->name);
-    return NULL;
-  }
-
-  cli_listener_add_tail(&cli_listeners, l);
-
-  return l;
+  if (chmod(path_control_socket, 0660) < 0)
+    die("chmod: %m");
 }
 
-static void
-cli_deafen(struct cli_listener *l)
-{
-  rfree(l->s);
-  unlink(l->config->name);
-  cli_listener_rem_node(&cli_listeners, l);
-  mb_free(l);
-}
-
-static void
-cli_init_unix(uid_t use_uid, gid_t use_gid)
-{
-  ASSERT_DIE(main_control_socket_config == NULL);
-
-  main_control_socket_config = &initial_control_socket_config;
-  main_control_socket_config->uid = use_uid;
-  main_control_socket_config->gid = use_gid;
-
-  ASSERT_DIE(main_control_socket == NULL);
-  main_control_socket = cli_listen(main_control_socket_config);
-  if (!main_control_socket)
-    die("Won't run without control socket");
-}
-
-static void
-cli_preconfig(struct config *c)
-{
-  if (!main_control_socket_config)
-    return;
-
-  struct cli_config *ccf = mb_alloc(cli_pool, sizeof *ccf);
-  memcpy(ccf, main_control_socket_config, sizeof *ccf);
-  ccf->n = (struct cli_config_node) {};
-  ccf->config = c;
-  cli_config_add_tail(&c->cli, ccf);
-}
-
-static void
-cli_commit(struct config *new, struct config *old)
-{
-  if (new->shutdown)
-  {
-    /* Keep the main CLI throughout the shutdown */
-    initial_control_socket_config.config = new;
-    main_control_socket->config = &initial_control_socket_config;
-  }
-
-  WALK_TLIST(cli_config, c, &new->cli)
-  {
-    _Bool seen = 0;
-    WALK_TLIST(cli_listener, l, &cli_listeners)
-      if (l->config->config != new)
-	if (!strcmp(l->config->name, c->name))
-	{
-	  ASSERT_DIE(l->config->config == old);
-	  l->config = c;
-	  seen = 1;
-	  break;
-	}
-
-    if (!seen)
-      cli_listen(c);
-  }
-
-  WALK_TLIST_DELSAFE(cli_listener, l, &cli_listeners)
-    if (l->config->config != new)
-      cli_deafen(l);
-}
-
-
 /*
  *	PID file
  */
@@ -750,7 +622,7 @@ void
 sysdep_shutdown_done(void)
 {
   unlink_pid_file();
-  cli_deafen(main_control_socket);
+  unlink(path_control_socket);
   log_msg(L_FATAL "Shutdown completed");
   exit(0);
 }
@@ -1023,15 +895,12 @@ main(int argc, char **argv)
   io_init();
   rt_init();
   if_init();
-  mpls_init();
 //  roa_init();
   config_init();
 
   uid_t use_uid = get_uid(use_user);
   gid_t use_gid = get_gid(use_group);
 
-  cli_init();
-
   if (!parse_and_exit)
   {
     test_old_bird(path_control_socket);
